module Xlib 
	( module X
	, module Xlib
	, free
	) where


import Xlib_StdDIS
import Monad( zipWithM_ )
import Hugs.Word
import Hugs.Prelude hiding (ForeignObj)
import Ptr
import ForeignPtr

import X

type ForeignObj' = Ptr ()

type Pixel        = Word32
type Position     = Int32
type Dimension    = Word32
type ScreenNumber = Word32
type Byte         = Char
type Buffer       = Int


newtype Display    = Display    Addr
newtype Screen     = Screen     Addr
newtype Visual     = Visual     Addr
newtype FontStruct = FontStruct Addr
newtype Region     = Region     ForeignObj


type GC = Addr

type Point = 
  ( Position  -- x
  , Position  -- y
  )

type Segment = 
 ( Position -- x1
 , Position -- y1
 , Position -- x2
 , Position -- y2
 )

type Rectangle = 
 ( Position  -- x
 , Position  -- y
 , Dimension -- width
 , Dimension -- height
 )

type Arc =
 ( Position  
 , Position  
 , Dimension 
 , Dimension 
 , Int       
 , Int       
 )

type Color =
 ( Pixel 
 , Word16 
 , Word16 
 , Word16 
 , Word8 
 )

-- don't forget to use %end free(arg?) in conjunction with these guys
type ListPoint       	= [Point]
type ListRectangle   	= [Rectangle]	     
type ListArc        	= [Arc]	     
type ListSegment       = [Segment]	     
type ListColor	     	= [Color]	     
type ListPixel	     	= [Pixel]
type ListWindow	     	= [Window]
-- AC, 1/9/2000: Try to define types and marshalling code for Atom lists:
type ListAtom	     	= [Atom]




marshallAddrList :: (Int -> IO Addr) -> (Addr -> Int -> a -> IO ()) -> [a] -> IO (Addr, Int)
marshallAddrList alloc write as = do
  let l = length as
  arr <- alloc l
  zipWithM_ (write arr) [0..] as
  return (arr, l)

unmarshallAddrList :: (Addr -> Int -> IO a) -> (Addr, Int) -> IO [a]
unmarshallAddrList read (ptr,l) = mapM (read ptr) [0..l-1]

writeAddr :: Addr -> Int -> Addr -> IO ()
writeAddr s i v =
  prim_writeAddr s i v
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeAddr" prim_writeAddr :: Addr -> Int -> Addr -> IO ()
 
readAddr :: Addr -> Int -> IO Addr
readAddr s i =
  prim_readAddr s i
  >>= \  gc_res1  ->
  (return (gc_res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readAddr" prim_readAddr :: Addr -> Int -> IO (Addr)

allocAddr :: Int -> IO Addr
allocAddr arg1 =
  prim_allocAddr arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocAddr" prim_allocAddr :: Int -> IO (Addr)
  

writePoint :: Addr -> Int -> Point -> IO ()
writePoint s i gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg4) ->
  case ( int32ToInt  gc_arg2) of { gc_arg3 ->
  case ( int32ToInt  gc_arg4) of { gc_arg5 ->
  prim_writePoint s i gc_arg3 gc_arg5}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writePoint" prim_writePoint :: Addr -> Int -> Int -> Int -> IO ()
readPoint :: Addr -> Int -> IO Point
readPoint s i =
  prim_readPoint s i
  >>= \ gc_result ->
  access_prim_readPoint_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_readPoint_gc_res4 gc_result >>= \ gc_res4 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return ((gc_res1,gc_res3)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readPoint" prim_readPoint :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readPoint_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readPoint_gc_res4 :: Ptr a1 -> IO (Int)
allocPoints :: Int -> IO Addr
allocPoints arg1 =
  prim_allocPoints arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocPoints" prim_allocPoints :: Int -> IO (Addr)

writeRectangle :: Addr -> Int -> Rectangle -> IO ()
writeRectangle s i gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg4,gc_arg6,gc_arg7) ->
  case ( int32ToInt  gc_arg2) of { gc_arg3 ->
  case ( int32ToInt  gc_arg4) of { gc_arg5 ->
  prim_writeRectangle s i gc_arg3 gc_arg5 gc_arg6 gc_arg7}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeRectangle" prim_writeRectangle :: Addr -> Int -> Int -> Int -> Word32 -> Word32 -> IO ()
readRectangle :: Addr -> Int -> IO Rectangle
readRectangle s i =
  prim_readRectangle s i
  >>= \ gc_result ->
  access_prim_readRectangle_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_readRectangle_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_readRectangle_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_readRectangle_gc_res6 gc_result >>= \ gc_res6 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return ((gc_res1,gc_res3,gc_res5,gc_res6)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readRectangle" prim_readRectangle :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readRectangle_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readRectangle_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readRectangle_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readRectangle_gc_res6 :: Ptr a1 -> IO (Word32)
allocRectangles :: Int -> IO Addr
allocRectangles arg1 =
  prim_allocRectangles arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocRectangles" prim_allocRectangles :: Int -> IO (Addr)

writeArc :: Addr -> Int -> Arc -> IO ()
writeArc s i gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg4,gc_arg6,gc_arg7,gc_arg8,gc_arg9) ->
  case ( int32ToInt  gc_arg2) of { gc_arg3 ->
  case ( int32ToInt  gc_arg4) of { gc_arg5 ->
  prim_writeArc s i gc_arg3 gc_arg5 gc_arg6 gc_arg7 gc_arg8 gc_arg9}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeArc" prim_writeArc :: Addr -> Int -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> IO ()
readArc :: Addr -> Int -> IO Arc
readArc s i =
  prim_readArc s i
  >>= \ gc_result ->
  access_prim_readArc_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_readArc_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_readArc_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_readArc_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_readArc_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_readArc_gc_res8 gc_result >>= \ gc_res8 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return ((gc_res1,gc_res3,gc_res5,gc_res6,gc_res7,gc_res8)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readArc" prim_readArc :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res6 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readArc_gc_res8 :: Ptr a1 -> IO (Int)
allocArcs :: Int -> IO Addr
allocArcs arg1 =
  prim_allocArcs arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocArcs" prim_allocArcs :: Int -> IO (Addr)

writeSegment :: Addr -> Int -> Segment -> IO ()
writeSegment s i gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg4,gc_arg6,gc_arg8) ->
  case ( int32ToInt  gc_arg2) of { gc_arg3 ->
  case ( int32ToInt  gc_arg4) of { gc_arg5 ->
  case ( int32ToInt  gc_arg6) of { gc_arg7 ->
  case ( int32ToInt  gc_arg8) of { gc_arg9 ->
  prim_writeSegment s i gc_arg3 gc_arg5 gc_arg7 gc_arg9}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeSegment" prim_writeSegment :: Addr -> Int -> Int -> Int -> Int -> Int -> IO ()
readSegment :: Addr -> Int -> IO Segment
readSegment s i =
  prim_readSegment s i
  >>= \ gc_result ->
  access_prim_readSegment_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_readSegment_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_readSegment_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_readSegment_gc_res8 gc_result >>= \ gc_res8 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  let gc_res5 = ( intToInt32  (gc_res6)) in
  let gc_res7 = ( intToInt32  (gc_res8)) in
  (return ((gc_res1,gc_res3,gc_res5,gc_res7)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readSegment" prim_readSegment :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readSegment_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readSegment_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readSegment_gc_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readSegment_gc_res8 :: Ptr a1 -> IO (Int)
allocSegments :: Int -> IO Addr
allocSegments arg1 =
  prim_allocSegments arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocSegments" prim_allocSegments :: Int -> IO (Addr)

writeColor :: Addr -> Int -> Color -> IO ()
writeColor s i gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg3,gc_arg5,gc_arg7,gc_arg9) ->
  case ( word16ToWord32  gc_arg3) of { gc_arg4 ->
  case ( word16ToWord32  gc_arg5) of { gc_arg6 ->
  case ( word16ToWord32  gc_arg7) of { gc_arg8 ->
  case ( word8ToWord32   gc_arg9) of { gc_arg10 ->
  prim_writeColor s i gc_arg2 gc_arg4 gc_arg6 gc_arg8 gc_arg10}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeColor" prim_writeColor :: Addr -> Int -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO ()
readColor :: Addr -> Int -> IO Color
readColor s i =
  prim_readColor s i
  >>= \ gc_result ->
  access_prim_readColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_readColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_readColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_readColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_readColor_gc_res9 gc_result >>= \ gc_res9 ->
  let gc_res2 = ( word32ToWord16  (gc_res3)) in
  let gc_res4 = ( word32ToWord16  (gc_res5)) in
  let gc_res6 = ( word32ToWord16  (gc_res7)) in
  let gc_res8 = ( word32ToWord8   (gc_res9)) in
  (return ((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readColor" prim_readColor :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_readColor_gc_res9 :: Ptr a1 -> IO (Word32)
allocColors :: Int -> IO Addr
allocColors arg1 =
  prim_allocColors arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocColors" prim_allocColors :: Int -> IO (Addr)


marshallWord32List :: [Word32] -> IO (Addr, Int)
marshallWord32List as = do
  let l = length as
  arr <- allocWord32Star l
  zipWithM_ (writeWord32 arr) [0..] as
  return (arr, l)

unmarshallWord32List :: (Addr, Int) -> IO [Word32]
unmarshallWord32List (ptr,l) = mapM (readWord32 ptr) [0..l-1]

writeWord32 :: Addr -> Int -> Word32 -> IO ()
writeWord32 s i v =
  prim_writeWord32 s i v
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_writeWord32" prim_writeWord32 :: Addr -> Int -> Word32 -> IO ()
 
readWord32 :: Addr -> Int -> IO Word32
readWord32 s i =
  prim_readWord32 s i
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_readWord32" prim_readWord32 :: Addr -> Int -> IO (Word32)

allocWord32Star :: Int -> IO Addr
allocWord32Star arg1 =
  prim_allocWord32Star arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocWord32Star" prim_allocWord32Star :: Int -> IO (Addr)
  

allPlanes :: Pixel
allPlanes =
  unsafePerformIO(
    prim_allPlanes
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allPlanes" prim_allPlanes :: IO (Word32)
blackPixel :: Display -> ScreenNumber -> Pixel
blackPixel gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_blackPixel arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_blackPixel" prim_blackPixel :: Addr -> Word32 -> IO (Word32)
whitePixel :: Display -> ScreenNumber -> Pixel
whitePixel gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_whitePixel arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_whitePixel" prim_whitePixel :: Addr -> Word32 -> IO (Word32)

-- This may vary from one execution to another but I believe it 
-- is constant during any given execution and so it can be made PURE
-- without breaking referential transparency.
--
-- Note: underneath the opaque name, it turns out that this
-- is the file descriptor.  You need to know this if you want to
-- use select.
connectionNumber :: Display -> Int
connectionNumber gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_connectionNumber arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_connectionNumber" prim_connectionNumber :: Addr -> IO (Int)

defaultColormap :: Display -> ScreenNumber -> Colormap
defaultColormap gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultColormap arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultColormap" prim_defaultColormap :: Addr -> Word32 -> IO (Word32)

-- %fun XListDepths :: Display -> ScreenNumber -> ListInt using res1 = XListDepths(arg1,arg2,&res1_size)

defaultGC :: Display -> ScreenNumber -> GC
defaultGC gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultGC arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultGC" prim_defaultGC :: Addr -> Word32 -> IO (Addr)
defaultDepth :: Display -> ScreenNumber -> Int
defaultDepth gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultDepth arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultDepth" prim_defaultDepth :: Addr -> Word32 -> IO (Int)
defaultScreen :: Display -> ScreenNumber
defaultScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultScreen" prim_defaultScreen :: Addr -> IO (Word32)
defaultScreenOfDisplay :: Display -> Screen
defaultScreenOfDisplay gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultScreenOfDisplay arg1
    >>= \  res1  ->
    (return ((Screen res1)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultScreenOfDisplay" prim_defaultScreenOfDisplay :: Addr -> IO (Addr)
displayHeight :: Display -> ScreenNumber -> Int
displayHeight gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayHeight arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayHeight" prim_displayHeight :: Addr -> Word32 -> IO (Int)
displayHeightMM :: Display -> ScreenNumber -> Int
displayHeightMM gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayHeightMM arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayHeightMM" prim_displayHeightMM :: Addr -> Word32 -> IO (Int)
displayWidth :: Display -> ScreenNumber -> Int
displayWidth gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayWidth arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayWidth" prim_displayWidth :: Addr -> Word32 -> IO (Int)
displayWidthMM :: Display -> ScreenNumber -> Int
displayWidthMM gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayWidthMM arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayWidthMM" prim_displayWidthMM :: Addr -> Word32 -> IO (Int)
maxRequestSize :: Display -> Int
maxRequestSize gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_maxRequestSize arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_maxRequestSize" prim_maxRequestSize :: Addr -> IO (Int)
displayMotionBufferSize :: Display -> Int
displayMotionBufferSize gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayMotionBufferSize arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayMotionBufferSize" prim_displayMotionBufferSize :: Addr -> IO (Int)
--Disnae exist in X11R5 %fun XExtendedMaxRequestSize :: Display -> Int
resourceManagerString :: Display -> String
resourceManagerString gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_resourceManagerString arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_resourceManagerString" prim_resourceManagerString :: Addr -> IO (Addr)
screenResourceString :: Screen -> String
screenResourceString gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_screenResourceString arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenResourceString" prim_screenResourceString :: Addr -> IO (Addr)
displayString :: Display -> String
displayString gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayString arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayString" prim_displayString :: Addr -> IO (Addr)
imageByteOrder :: Display -> Int
imageByteOrder gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_imageByteOrder arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_imageByteOrder" prim_imageByteOrder :: Addr -> IO (Int)
protocolRevision :: Display -> Int
protocolRevision gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_protocolRevision arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_protocolRevision" prim_protocolRevision :: Addr -> IO (Int)
protocolVersion :: Display -> Int
protocolVersion gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_protocolVersion arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_protocolVersion" prim_protocolVersion :: Addr -> IO (Int)
serverVendor :: Display -> String
serverVendor gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_serverVendor arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_serverVendor" prim_serverVendor :: Addr -> IO (Addr)
--Disnae exist: %fun XServerRelease          :: Display -> Int    
screenCount :: Display -> Int
screenCount gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_screenCount arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenCount" prim_screenCount :: Addr -> IO (Int)
defaultVisual :: Display -> ScreenNumber -> Visual
defaultVisual gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultVisual arg1 arg2
    >>= \  res1  ->
    (return ((Visual res1)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultVisual" prim_defaultVisual :: Addr -> Word32 -> IO (Addr)
displayCells :: Display -> ScreenNumber -> Int
displayCells gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayCells arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayCells" prim_displayCells :: Addr -> Word32 -> IO (Int)
displayPlanes :: Display -> ScreenNumber -> Int
displayPlanes gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayPlanes arg1 arg2
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayPlanes" prim_displayPlanes :: Addr -> Word32 -> IO (Int)
screenOfDisplay :: Display -> ScreenNumber -> Screen
screenOfDisplay gc_arg1 arg2 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_screenOfDisplay arg1 arg2
    >>= \  res1  ->
    (return ((Screen res1)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenOfDisplay" prim_screenOfDisplay :: Addr -> Word32 -> IO (Addr)
defaultRootWindow :: Display -> Window
defaultRootWindow gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_defaultRootWindow arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultRootWindow" prim_defaultRootWindow :: Addr -> IO (Word32)

-- The following are believed to be order dependent

rootWindow :: Display -> ScreenNumber -> IO Window
rootWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_rootWindow arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rootWindow" prim_rootWindow :: Addr -> Word32 -> IO (Word32)
qLength :: Display -> IO Int
qLength gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_qLength arg1
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_qLength" prim_qLength :: Addr -> IO (Int)
     		    	     
noOp :: Display -> IO ()
noOp gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_noOp arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_noOp" prim_noOp :: Addr -> IO ()

openDisplay :: String -> IO Display
openDisplay gc_arg1 =
  (marshall_string_ gc_arg1) >>= \ (arg1) ->
  prim_openDisplay arg1
  >>= \ gc_result ->
  access_prim_openDisplay_res1 gc_result >>= \ res1 ->
  access_prim_openDisplay_gc_failed gc_result >>= \ gc_failed ->
  access_prim_openDisplay_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((Display res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_openDisplay" prim_openDisplay :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_openDisplay_res1 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_openDisplay_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_openDisplay_gc_failstring :: Ptr a1 -> IO (Addr)

closeDisplay :: Display -> IO ()
closeDisplay gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_closeDisplay arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_closeDisplay" prim_closeDisplay :: Addr -> IO ()

-- I'm not sure why I added this since I don't have any of the related
-- functions.
rmInitialize :: IO ()
rmInitialize =
  prim_rmInitialize
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rmInitialize" prim_rmInitialize :: IO ()

-- %fun XGetDefault :: Display -> String -> String -> IO ()

autoRepeatOff :: Display -> IO ()
autoRepeatOff gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_autoRepeatOff arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_autoRepeatOff" prim_autoRepeatOff :: Addr -> IO ()
autoRepeatOn :: Display -> IO ()
autoRepeatOn gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_autoRepeatOn arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_autoRepeatOn" prim_autoRepeatOn :: Addr -> IO ()
bell :: Display -> Int -> IO ()
bell gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_bell arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_bell" prim_bell :: Addr -> Int -> IO ()
setCloseDownMode :: Display -> CloseDownMode -> IO ()
setCloseDownMode gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_setCloseDownMode arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setCloseDownMode" prim_setCloseDownMode :: Addr -> Int -> IO ()
lastKnownRequestProcessed :: Display -> IO Int
lastKnownRequestProcessed gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_lastKnownRequestProcessed arg1
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_lastKnownRequestProcessed" prim_lastKnownRequestProcessed :: Addr -> IO (Int)

getInputFocus :: Display -> IO (Window,FocusMode)
getInputFocus gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_getInputFocus arg1
  >>= \ gc_result ->
  access_prim_getInputFocus_res1 gc_result >>= \ res1 ->
  access_prim_getInputFocus_res2 gc_result >>= \ res2 ->
  (return ((res1,res2)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_getInputFocus" prim_getInputFocus :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getInputFocus_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getInputFocus_res2 :: Ptr a1 -> IO (Int)

setInputFocus :: Display -> Window -> FocusMode -> Time -> IO ()
setInputFocus gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_setInputFocus arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setInputFocus" prim_setInputFocus :: Addr -> Word32 -> Int -> Word32 -> IO ()

-- XAllocID omitted
-- XKillClient omitted
-- XFetchName omitted
-- XGetKeyboardControl omitted
-- XChangeKeyboardControl omitted
-- XChangeKeyboardMapping omitted
-- XChangePointerControl omitted

grabButton :: Display -> Button -> ButtonMask -> Window -> Bool -> EventMask -> GrabMode -> GrabMode -> Window -> Cursor -> IO ()
grabButton gc_arg1 arg2 arg3 arg4 gc_arg2 arg6 arg7 arg8 arg9 arg10 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg5) ->
  prim_grabButton arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_grabButton" prim_grabButton :: Addr -> Word32 -> Word32 -> Word32 -> Int -> Word32 -> Int -> Int -> Word32 -> Word32 -> IO ()
ungrabButton :: Display -> Button -> ButtonMask -> Window -> IO ()
ungrabButton gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_ungrabButton arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ungrabButton" prim_ungrabButton :: Addr -> Word32 -> Word32 -> Word32 -> IO ()

grabPointer :: Display -> Window -> Bool -> EventMask -> GrabMode -> GrabMode -> Window -> Cursor -> Time -> IO GrabStatus
grabPointer gc_arg1 arg2 gc_arg2 arg4 arg5 arg6 arg7 arg8 arg9 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg3) ->
  prim_grabPointer arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_grabPointer" prim_grabPointer :: Addr -> Word32 -> Int -> Word32 -> Int -> Int -> Word32 -> Word32 -> Word32 -> IO (Int)
ungrabPointer :: Display -> Time -> IO ()
ungrabPointer gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_ungrabPointer arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ungrabPointer" prim_ungrabPointer :: Addr -> Word32 -> IO ()

grabKey :: Display -> KeyCode -> ButtonMask -> Window -> Bool -> GrabMode -> GrabMode -> IO ()
grabKey gc_arg1 arg2 arg3 arg4 gc_arg2 arg6 arg7 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg5) ->
  prim_grabKey arg1 arg2 arg3 arg4 arg5 arg6 arg7}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_grabKey" prim_grabKey :: Addr -> Char -> Word32 -> Word32 -> Int -> Int -> Int -> IO ()
ungrabKey :: Display -> KeyCode -> ButtonMask -> Window -> IO ()
ungrabKey gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_ungrabKey arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ungrabKey" prim_ungrabKey :: Addr -> Char -> Word32 -> Word32 -> IO ()

grabKeyboard :: Display -> Window -> Bool -> GrabMode -> GrabMode -> Time -> IO GrabStatus
grabKeyboard gc_arg1 arg2 gc_arg2 arg4 arg5 arg6 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg3) ->
  prim_grabKeyboard arg1 arg2 arg3 arg4 arg5 arg6
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_grabKeyboard" prim_grabKeyboard :: Addr -> Word32 -> Int -> Int -> Int -> Word32 -> IO (Int)
ungrabKeyboard :: Display -> Time -> IO ()
ungrabKeyboard gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_ungrabKeyboard arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ungrabKeyboard" prim_ungrabKeyboard :: Addr -> Word32 -> IO ()

grabServer :: Display -> IO ()
grabServer gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_grabServer arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_grabServer" prim_grabServer :: Addr -> IO ()
ungrabServer :: Display -> IO ()
ungrabServer gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_ungrabServer arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ungrabServer" prim_ungrabServer :: Addr -> IO ()

-- XChangeActivePointerGrab omitted

-- XFree omitted
-- XFreeStringList omitted

queryBestTile :: Display -> Drawable -> Dimension -> Dimension -> IO (Dimension,Dimension)
queryBestTile gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryBestTile arg1 arg2 arg3 arg4
  >>= \ gc_result ->
  access_prim_queryBestTile_res1 gc_result >>= \ res1 ->
  access_prim_queryBestTile_res2 gc_result >>= \ res2 ->
  access_prim_queryBestTile_gc_failed gc_result >>= \ gc_failed ->
  access_prim_queryBestTile_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((res1,res2)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryBestTile" prim_queryBestTile :: Addr -> Word32 -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestTile_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestTile_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestTile_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestTile_gc_failstring :: Ptr a1 -> IO (Addr)

queryBestStipple :: Display -> Drawable -> Dimension -> Dimension -> IO (Dimension,Dimension)
queryBestStipple gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryBestStipple arg1 arg2 arg3 arg4
  >>= \ gc_result ->
  access_prim_queryBestStipple_res1 gc_result >>= \ res1 ->
  access_prim_queryBestStipple_res2 gc_result >>= \ res2 ->
  access_prim_queryBestStipple_gc_failed gc_result >>= \ gc_failed ->
  access_prim_queryBestStipple_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((res1,res2)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryBestStipple" prim_queryBestStipple :: Addr -> Word32 -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestStipple_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestStipple_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestStipple_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestStipple_gc_failstring :: Ptr a1 -> IO Addr

queryBestCursor :: Display -> Drawable -> Dimension -> Dimension -> IO (Dimension,Dimension)
queryBestCursor gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryBestCursor arg1 arg2 arg3 arg4
  >>= \ gc_result ->
  access_prim_queryBestCursor_res1 gc_result >>= \ res1 ->
  access_prim_queryBestCursor_res2 gc_result >>= \ res2 ->
  access_prim_queryBestCursor_gc_failed gc_result >>= \ gc_failed ->
  access_prim_queryBestCursor_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((res1,res2)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryBestCursor" prim_queryBestCursor :: Addr -> Word32 -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestCursor_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestCursor_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestCursor_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestCursor_gc_failstring :: Ptr a1 -> IO (Addr)

queryBestSize :: Display -> Drawable -> QueryBestSizeClass -> Dimension -> Dimension -> IO (Dimension,Dimension)
queryBestSize gc_arg1 arg2 arg3 arg4 arg5 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryBestSize arg1 arg2 arg3 arg4 arg5
  >>= \ gc_result ->
  access_prim_queryBestSize_res1 gc_result >>= \ res1 ->
  access_prim_queryBestSize_res2 gc_result >>= \ res2 ->
  access_prim_queryBestSize_gc_failed gc_result >>= \ gc_failed ->
  access_prim_queryBestSize_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((res1,res2)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryBestSize" prim_queryBestSize :: Addr -> Word32 -> Int -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestSize_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestSize_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestSize_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryBestSize_gc_failstring :: Ptr a1 -> IO (Addr)

-- Note: Returns false if pointer not in window w (and win_x = win_y = 0)
-- ToDo: more effective use of Maybes?
queryPointer :: Display -> Window -> IO (Bool,Window,Window,Int,Int,Int,Int,Modifier)
queryPointer gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryPointer arg1 arg2
  >>= \ gc_result ->
  access_prim_queryPointer_res1 gc_result >>= \ res1 ->
  access_prim_queryPointer_res2 gc_result >>= \ res2 ->
  access_prim_queryPointer_res3 gc_result >>= \ res3 ->
  access_prim_queryPointer_res4 gc_result >>= \ res4 ->
  access_prim_queryPointer_res5 gc_result >>= \ res5 ->
  access_prim_queryPointer_res6 gc_result >>= \ res6 ->
  access_prim_queryPointer_res7 gc_result >>= \ res7 ->
  access_prim_queryPointer_res8 gc_result >>= \ res8 ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return ((gc_res1,res2,res3,res4,res5,res6,res7,res8)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryPointer" prim_queryPointer :: Addr -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res5 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryPointer_res8 :: Ptr a1 -> IO (Word32)

-- XSetSelectionOwner omitted

-- XOpenOM omitted
-- XCloseOM omitted
-- XSetOMValues omitted
-- XGetOMValues omitted
-- DisplayOfOM omitted
-- XLocaleOfOM omitted

-- XCreateOC omitted
-- XDestroyOC omitted
-- XOMOfOC omitted
-- XSetOCValues omitted
-- XGetOCValues omitted

-- XVaCreateNestedList omitted

----------------------------------------------------------------
-- Error reporting
----------------------------------------------------------------

displayName :: String -> String
displayName gc_arg1 =
  unsafePerformIO(
    (marshall_string_ gc_arg1) >>= \ (arg1) ->
    prim_displayName arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayName" prim_displayName :: Addr -> IO (Addr)

-- type ErrorHandler   = Display -> ErrorEvent -> IO Int
-- %dis errorHandler x = (stable x)
-- 
-- type IOErrorHandler = Display ->                IO Int
-- %dis ioErrorHandler x = (stable x)

-- Sadly, this code doesn't work because hugs->runIO creates a fresh
-- stack of exception handlers so the exception gets thrown to the 
-- wrong place.
-- 
-- %C
-- % static HugsStablePtr ioErrorHandlerPtr; 
-- % 
-- % int genericIOErrorHandler(Display *d)
-- % {
-- %     if (ioErrorHandlerPtr >= 0) {
-- %     	  hugs->putStablePtr(ioErrorHandlerPtr);
-- %     	  hugs->putAddr(d);  
-- %     	  if (hugs->runIO(1)) { /* exitWith value returned */
-- %     	   return hugs->getInt();
-- %     	  } else {
-- %     	   return hugs->getWord();
-- %     	  }
-- %     }
-- %     return 1;
-- % }

-- Here's what we might do instead.  The two error handlers set flags
-- when they fire and every single call to X contains the line:
--
--   %fail { errorFlags != 0 } { XError(errorFlags) }
--
-- This really sucks.
-- Oh, and it won't even work with IOErrors since they terminate
-- the process if the handler returns.  I don't know what the hell they
-- think they're doing taking it upon themselves to terminate MY
-- process when THEIR library has a problem but I don't think anyone 
-- ever accused X of being well-designed.
-- 
-- % static int genericIOErrorHandler(Display *d)
-- % {
-- %     if (ioErrorHandlerPtr >= 0) {
-- %     	  hugs->putStablePtr(ioErrorHandlerPtr);
-- %     	  hugs->putAddr(d);  
-- %     	  if (hugs->runIO(1)) { /* exitWith value returned */
-- %     	   return hugs->getInt();
-- %     	  } else {
-- %     	   return hugs->getWord();
-- %     	  }
-- %     }
-- %     return 1;
-- % }


-- HN 2001-02-06
-- Moved to auxiliaries.c to make it easier to use the inlining option.
-- -- Sigh, for now we just use an error handler that prints an error
-- -- message on the screen
-- %C
-- % static int defaultErrorHandler(Display *d, XErrorEvent *ev)
-- % {
-- % 	  char buffer[1000];
-- % 	  XGetErrorText(d,ev->error_code,buffer,1000);
-- % 	  printf("Error: %s\n", buffer);
-- % 	  return 0;
-- % }


setDefaultErrorHandler :: IO ()
setDefaultErrorHandler =
  prim_setDefaultErrorHandler
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setDefaultErrorHandler" prim_setDefaultErrorHandler :: IO ()


-- %fun XSetIOErrorHandler :: IOErrorHandler -> IO IOErrorHandler
-- %fun XSetErrorHandler   :: ErrorHandler   -> IO ErrorHandler

-- XGetErrorDatabaseText omitted
-- XGetErrorText omitted

-- ----------------------------------------------------------------
-- -- Buffers
-- ----------------------------------------------------------------
-- 
-- -- OLD: Would arrays be more appropriate?
-- --
-- -- IMPURE void	XStoreBytes(display, bytes, nbytes)
-- -- IN Display*		display
-- -- VAR Int			nbytes
-- -- IN list[nbytes] Byte	bytes
-- -- 
-- -- IMPURE list[nbytes] Byte	XFetchBytes(display, &nbytes)
-- -- IN Display*	display
-- -- VAR Int		nbytes
-- -- 
-- -- IMPURE void	XStoreBuffer(display, bytes, nbytes, buffer)
-- -- IN Display*		display
-- -- VAR Int			nbytes
-- -- IN list[nbytes] Byte	bytes
-- -- IN Buffer		buffer
-- -- 
-- -- IMPURE list[nbytes] Byte	XFetchBuffer(display, &nbytes, buffer)
-- -- IN Display*	display
-- -- VAR Int		nbytes
-- -- IN Buffer	buffer
-- -- 
-- -- IMPURE void	XRotateBuffers(display, rotate)
-- -- IN Display*	display
-- -- VAR Int		rotate

----------------------------------------------------------------
-- Extensions
----------------------------------------------------------------

-- ToDo: Use XFreeExtensionList
-- %fun XListExtensions :: Display -> IO ListString using res1 = XListExtensions(arg1,&res1_size)

-- %errfun False XQueryExtension :: Display -> String -> IO (Int,Int,Int) using res4 = XQueryExtension(arg1,arg2,&res1,&res2,&res3)->(res1,res2,res3)
-- %fun XInitExtensions :: Display -> String -> IO XExtCodes
-- %fun XAddExtensions  :: Display ->           IO XExtCodes

-- XAddToExtensionList omitted
-- XFindOnExtensionList omitted
-- XEHeadOfExtensionList omitted

----------------------------------------------------------------
-- Hosts
----------------------------------------------------------------

-- ToDo: operations to construct and destruct an XHostAddress

-- %fun XAddHost :: Display -> XHostAddress -> IO ()
-- %fun XRemoveHost :: Display -> XHostAddress -> IO ()
-- 
-- %fun XAddHosts    :: Display -> ListXHostAddress -> IO () using XAddHosts(arg1,arg2,arg2_size) 
-- %fun XRemoveHosts :: Display -> ListXHostAddress -> IO () using XRemoveHosts(arg1,arg2,arg2_size) 
-- 
-- # Uses %prim to let us call XFree
-- %prim XListHosts :: Display -> IO (ListXHostAddress, Bool)
-- Bool state;
-- Int r_size;
-- XHostAddress* r = XListHosts(arg1,&r_size,&state);
-- %update(r,state);
-- XFree(r);
-- return;

-- %fun XEnableAccessControl  :: Display -> IO ()
-- %fun XDisableAccessControl :: Display -> IO ()
-- %fun XSetAccessControl     :: Display -> Access -> IO ()


----------------------------------------------------------------
-- Events
----------------------------------------------------------------

type   QueuedMode   = Int
queuedAlready :: QueuedMode
queuedAlready =
  unsafePerformIO(
    prim_queuedAlready
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queuedAlready" prim_queuedAlready :: IO (Int)
queuedAfterFlush :: QueuedMode
queuedAfterFlush =
  unsafePerformIO(
    prim_queuedAfterFlush
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queuedAfterFlush" prim_queuedAfterFlush :: IO (Int)
queuedAfterReading :: QueuedMode
queuedAfterReading =
  unsafePerformIO(
    prim_queuedAfterReading
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queuedAfterReading" prim_queuedAfterReading :: IO (Int)

-- Because of the way the corresponding C types are defined,
-- These "structs" are somewhat unusual - they omit fields which can
-- be found in more general structs.
-- For example, XAnyEvent omits type since it is in XEvent.
-- Therefore, to get the complete contents of an event one typically
-- writes:
--   do
--     ty <- get_XEvent e
--     (serial,send_event,display,window) <- get_XAnyEvent
--     window' <- get_XDestroyWindowEvent

type XEvent = 
 ( EventType
 )
type XEventPtr   = Addr

allocXEvent :: IO XEventPtr
allocXEvent =
  prim_allocXEvent
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocXEvent" prim_allocXEvent :: IO (Addr)

get_EventType :: XEventPtr -> IO EventType
get_EventType arg1 =
  prim_get_EventType arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_EventType" prim_get_EventType :: Addr -> IO (Word32)

get_Window :: XEventPtr -> IO Window
get_Window arg1 =
  prim_get_Window arg1
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_Window" prim_get_Window :: Addr -> IO (Word32)

-- %struct : XAnyEvent : XAnyEvent arg1
--   Int32     : serial            # # of last request processed by server
--   Bool      : send_event        # true if this came from a SendEvent request
--   Display   : display           # Display the event was read from
--   Window    : window            # window on which event was requested in event mask

type XKeyEvent =
 ( Window    -- root window that the event occured on 
 , Window    -- child window 
 , Time      -- milliseconds 
 , Int       -- pointer x, y coordinates in event window 
 , Int       -- 
 , Int       -- coordinates relative to root 
 , Int       -- 
 , Modifier  -- key or button mask 
 , KeyCode   -- detail 
 , Bool      -- same screen flag 
 )
type XKeyEventPtr   = Addr

type XButtonEvent = 
 ( Window    --	root window that the event occured on 
 , Window    --	child window 
 , Time      --	milliseconds 
 , Int       --	pointer x, y coordinates in event window 
 , Int       
 , Int       --	coordinates relative to root 
 , Int       
 , Modifier  --	key or button mask 
 , Button    --	detail 
 , Bool      --	same screen flag 
 )

get_KeyEvent :: XEventPtr -> IO XKeyEvent
get_KeyEvent arg1 =
  prim_get_KeyEvent arg1
  >>= \ gc_result ->
  access_prim_get_KeyEvent_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_get_KeyEvent_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_get_KeyEvent_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_get_KeyEvent_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_get_KeyEvent_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_get_KeyEvent_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_get_KeyEvent_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_get_KeyEvent_gc_res8 gc_result >>= \ gc_res8 ->
  access_prim_get_KeyEvent_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_get_KeyEvent_gc_res11 gc_result >>= \ gc_res11 ->
  (unmarshall_bool_ gc_res11) >>= \ gc_res10 ->
  (return ((gc_res1,gc_res2,gc_res3,gc_res4,gc_res5,gc_res6,gc_res7,gc_res8,gc_res9,gc_res10)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_KeyEvent" prim_get_KeyEvent :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res5 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res8 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res9 :: Ptr a1 -> IO (Char)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_KeyEvent_gc_res11 :: Ptr a1 -> IO (Int)

get_ButtonEvent :: XEventPtr -> IO XButtonEvent
get_ButtonEvent arg1 =
  prim_get_ButtonEvent arg1
  >>= \ gc_result ->
  access_prim_get_ButtonEvent_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_get_ButtonEvent_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_get_ButtonEvent_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_get_ButtonEvent_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_get_ButtonEvent_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_get_ButtonEvent_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_get_ButtonEvent_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_get_ButtonEvent_gc_res8 gc_result >>= \ gc_res8 ->
  access_prim_get_ButtonEvent_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_get_ButtonEvent_gc_res11 gc_result >>= \ gc_res11 ->
  (unmarshall_bool_ gc_res11) >>= \ gc_res10 ->
  (return ((gc_res1,gc_res2,gc_res3,gc_res4,gc_res5,gc_res6,gc_res7,gc_res8,gc_res9,gc_res10)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_ButtonEvent" prim_get_ButtonEvent :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res5 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res8 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res9 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ButtonEvent_gc_res11 :: Ptr a1 -> IO (Int)

get_MotionEvent :: XEventPtr -> IO XMotionEvent
get_MotionEvent arg1 =
  prim_get_MotionEvent arg1
  >>= \ gc_result ->
  access_prim_get_MotionEvent_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_get_MotionEvent_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_get_MotionEvent_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_get_MotionEvent_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_get_MotionEvent_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_get_MotionEvent_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_get_MotionEvent_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_get_MotionEvent_gc_res8 gc_result >>= \ gc_res8 ->
  access_prim_get_MotionEvent_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_get_MotionEvent_gc_res11 gc_result >>= \ gc_res11 ->
  (unmarshall_bool_ gc_res11) >>= \ gc_res10 ->
  (return ((gc_res1,gc_res2,gc_res3,gc_res4,gc_res5,gc_res6,gc_res7,gc_res8,gc_res9,gc_res10)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_MotionEvent" prim_get_MotionEvent :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res2 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res5 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res8 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res9 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_MotionEvent_gc_res11 :: Ptr a1 -> IO (Int)


type XMotionEvent =
 ( Window      -- root window that the event occured on 
 , Window      -- child window 
 , Time        -- milliseconds 
 , Int         -- pointer x, y coordinates in event window 
 , Int          
 , Int         -- coordinates relative to root 
 , Int          
 , Modifier    -- key or button mask 
 , NotifyMode  -- detail 
 , Bool        -- same screen flag 
 )


-- %struct : XCrossingEvent : XCrossingEvent arg1
--   Window       : root 	        # root window that the event occured on 
--   Window       : subwindow 	# child window 
--   Time         : time 		# milliseconds 
--   Int          : x 		# pointer x, y coordinates in event window 
--   Int          : y
--   Int          : x_root	 	# coordinates relative to root 
--   Int          : y_root
--   NotifyMode   : mode              
--   NotifyDetail : detail
--   Bool         : same_screen	# same screen flag
--   Bool         : focus		# boolean focus
--   Modifier     : state	        # key or button mask 
-- 
-- %struct : XFocusChangeEvent : XFocusChangeEvent arg1
--   NotifyMode   : mode
--   NotifyDetail : detail
-- 
-- -- omitted: should be translated into bitmaps
-- -- PURE void	getKeymapEvent(event)
-- -- IN XEvent*	event
-- -- OUT Window	window	 	= ((XKeymapEvent*)event)->window
-- -- OUT array[32] Char key_vector 	= ((XKeymapEvent*)event)->key_vector
-- -- RESULT: 

type XExposeEvent =
 ( Position	-- x	 	
 , Position	-- y	 	
 , Dimension	-- width	 	
 , Dimension	-- height	
 , Int		-- count	 	
 )

get_ExposeEvent :: XEventPtr -> IO XExposeEvent
get_ExposeEvent arg1 =
  prim_get_ExposeEvent arg1
  >>= \ gc_result ->
  access_prim_get_ExposeEvent_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_get_ExposeEvent_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_get_ExposeEvent_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_get_ExposeEvent_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_get_ExposeEvent_gc_res7 gc_result >>= \ gc_res7 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return ((gc_res1,gc_res3,gc_res5,gc_res6,gc_res7)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_ExposeEvent" prim_get_ExposeEvent :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ExposeEvent_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ExposeEvent_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ExposeEvent_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ExposeEvent_gc_res6 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ExposeEvent_gc_res7 :: Ptr a1 -> IO (Int)

-- %struct : XGraphicsExposeEvent : XGraphicsExposeEvent arg1
--   Position	: x	 	
--   Position	: y	 	
--   Dimension	: width	 	.
--   Dimension	: height	
--   Int		: count	 	
--   Int		: major_code 	
--   Int		: minor_code 	
-- 
-- %struct : XCirculateEvent : XCirculateEvent arg1
--   Window	: window	 	
--   Place		: place		
-- 
-- %struct : XConfigureEvent : XConfigureEvent arg1
--   Window	: window	
--   Position	: x	 	
--   Position	: y	 	
--   Dimension	: width	 	
--   Dimension	: height	
--   Dimension	: border_width 	
--   Window	: above	 	
--   Bool	        : override_redirect
-- 
-- %struct : XCreateWindowEvent : XCreateWindowEvent arg1
--   Window	: window	 	
--   Position	: x	 	
--   Position	: y	 	
--   Dimension	: width	 	
--   Dimension	: height	 	
--   Dimension	: border_width 	
--   Bool	        : override_redirect
-- 
-- %struct : XDestroyWindowEvent : XDestroyWindowEvent arg1
--   Window	: window
-- 
-- %struct : XGravityEvent : XGravityEvent arg1
--   Window	: window	
--   Position	: x	 	
--   Position	: y	 	
-- 
-- %struct : XMapEvent : XMapEvent arg1
--   Bool	        : override_redirect

type XMappingEvent =
 ( MappingRequest -- request	
 , KeyCode	  -- first_keycode
 , Int		  -- count	
 )

type XConfigureEvent =
 ( Position
 , Position
 , Dimension 
 , Dimension 
 )
get_ConfigureEvent :: XEventPtr -> IO XConfigureEvent
get_ConfigureEvent arg1 =
  prim_get_ConfigureEvent arg1
  >>= \ gc_result ->
  access_prim_get_ConfigureEvent_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_get_ConfigureEvent_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_get_ConfigureEvent_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_get_ConfigureEvent_gc_res6 gc_result >>= \ gc_res6 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return ((gc_res1,gc_res3,gc_res5,gc_res6)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_get_ConfigureEvent" prim_get_ConfigureEvent :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ConfigureEvent_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ConfigureEvent_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ConfigureEvent_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_get_ConfigureEvent_gc_res6 :: Ptr a1 -> IO (Word32)


-- %struct : XResizeRequestEvent : XResizeRequestEvent arg1
--   Dimension	: width	 
--   Dimension	: height	
-- 

-- %struct : XReparentEvent : XReparentEvent arg1
--   Window	: window	
--   Window	: parent	
--   Position	: x	 	
--   Position	: y	 	
--   Bool	        : override_redirect
-- 
-- %struct : XUnmapEvent : XUnmapEvent arg1
--   Window	: window	 	
--   Bool	        : from_configure	
-- 
-- %struct : XVisibilityEvent : XVisibilityEvent arg1
--   Visibility	: state
-- 
-- %struct : XCirculateRequestEvent : XCirculateRequestEvent arg1
--   Place	        : place
-- 
-- -- omitted because valuemask looks tricky
-- -- %struct : XConfigureRequestEvent : XConfigureRequestEvent arg1
-- --   Window	 : window	
-- --   Position	 : x		
-- --   Position	 : y		
-- --   Dimension	 : width	
-- --   Dimension	 : height	
-- --   Dimension	 : border_width	
-- --   Window	 : above	
-- --   StackingMethod : detail	
-- --   ???	         : valuemask	
-- 
-- %struct : XMapRequestEvent : XMapRequestEvent arg1
--   Window	: window
-- 
-- %struct : XColormapEvent : XColormapEvent arg1
--   Colormap		: colormap
--   Bool		        : new	  
--   ColormapNotification	: state   
-- 
-- -- getClientMessageEvent omitted
-- -- getPropertyEvent omitted
-- -- getSelectionClearEvent omitted
-- -- getSelectionRequestEvent omitted
-- -- getSelectionEvent omitted

-- functions

-- The following is useful if you want to do a read with timeout.
-- Returns True if timeout occurs.
waitForEvent :: Display -> Word32 -> IO Bool
waitForEvent gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_waitForEvent arg1 arg2
  >>= \  gc_res2  ->
  (unmarshall_bool_ gc_res2) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_waitForEvent" prim_waitForEvent :: Addr -> Word32 -> IO (Int)

-- The following is somewhat compatible with Win32's TimeGetTime
gettimeofday_in_milliseconds :: IO Integer
gettimeofday_in_milliseconds = do
  (sec,usec) <- gettimeofday_in_milliseconds_internal
  return (toInteger sec * 1000 + toInteger usec `div` 1000)

gettimeofday_in_milliseconds_internal :: IO (Word32,Word32)
gettimeofday_in_milliseconds_internal =
  prim_gettimeofday_in_milliseconds_internal
  >>= \ gc_result ->
  access_prim_gettimeofday_in_milliseconds_internal_res1 gc_result >>= \ res1 ->
  access_prim_gettimeofday_in_milliseconds_internal_res2 gc_result >>= \ res2 ->
  (return ((res1,res2)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_gettimeofday_in_milliseconds_internal" prim_gettimeofday_in_milliseconds_internal :: IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_gettimeofday_in_milliseconds_internal_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_gettimeofday_in_milliseconds_internal_res2 :: Ptr a1 -> IO (Word32)

flush :: Display -> IO ()
flush gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_flush arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_flush" prim_flush :: Addr -> IO ()
sync :: Display -> Bool -> IO ()
sync gc_arg1 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg2) ->
  prim_sync arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_sync" prim_sync :: Addr -> Int -> IO ()
pending :: Display -> IO Int
pending gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_pending arg1
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_pending" prim_pending :: Addr -> IO (Int)
eventsQueued :: Display -> QueuedMode -> IO Int
eventsQueued gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_eventsQueued arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_eventsQueued" prim_eventsQueued :: Addr -> Int -> IO (Int)
nextEvent :: Display -> XEventPtr -> IO ()
nextEvent gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_nextEvent arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_nextEvent" prim_nextEvent :: Addr -> Addr -> IO ()
allowEvents :: Display -> AllowEvents -> Time -> IO ()
allowEvents gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_allowEvents arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allowEvents" prim_allowEvents :: Addr -> Int -> Word32 -> IO ()

-- ToDo: XFree(res1) after constructing result
-- %fun XGetMotionEvents :: Display -> Window -> Time -> Time -> IO ListXTimeCoord
-- %code res1 = XGetMotionEvents(arg1,arg2,arg3,arg4,&res1_size)

selectInput :: Display -> Window -> EventMask -> IO ()
selectInput gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_selectInput arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_selectInput" prim_selectInput :: Addr -> Word32 -> Word32 -> IO ()
sendEvent :: Display -> Window -> Bool -> EventMask -> XEventPtr -> IO ()
sendEvent gc_arg1 arg2 gc_arg2 arg4 arg5 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg3) ->
  prim_sendEvent arg1 arg2 arg3 arg4 arg5
  >>= \ gc_result ->
  access_prim_sendEvent_gc_failed gc_result >>= \ gc_failed ->
  access_prim_sendEvent_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_sendEvent" prim_sendEvent :: Addr -> Word32 -> Int -> Word32 -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_sendEvent_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_sendEvent_gc_failstring :: Ptr a1 -> IO (Addr)

windowEvent :: Display -> Window -> EventMask -> XEventPtr -> IO ()
windowEvent gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_windowEvent arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_windowEvent" prim_windowEvent :: Addr -> Word32 -> Word32 -> Addr -> IO ()
checkWindowEvent :: Display -> Window -> EventMask -> XEventPtr -> IO Bool
checkWindowEvent gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_checkWindowEvent arg1 arg2 arg3 arg4
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_checkWindowEvent" prim_checkWindowEvent :: Addr -> Word32 -> Word32 -> Addr -> IO (Int)

maskEvent :: Display -> EventMask -> XEventPtr -> IO ()
maskEvent gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_maskEvent arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_maskEvent" prim_maskEvent :: Addr -> Word32 -> Addr -> IO ()
checkMaskEvent :: Display -> EventMask -> XEventPtr -> IO Bool
checkMaskEvent gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_checkMaskEvent arg1 arg2 arg3
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_checkMaskEvent" prim_checkMaskEvent :: Addr -> Word32 -> Addr -> IO (Int)
checkTypedEvent :: Display -> EventType -> XEventPtr -> IO Bool
checkTypedEvent gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_checkTypedEvent arg1 arg2 arg3
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_checkTypedEvent" prim_checkTypedEvent :: Addr -> Word32 -> Addr -> IO (Int)
checkTypedWindowEvent :: Display -> Window -> EventType -> XEventPtr -> IO Bool
checkTypedWindowEvent gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_checkTypedWindowEvent arg1 arg2 arg3 arg4
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_checkTypedWindowEvent" prim_checkTypedWindowEvent :: Addr -> Word32 -> Word32 -> Addr -> IO (Int)
putBackEvent :: Display -> XEventPtr -> IO ()
putBackEvent gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_putBackEvent arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_putBackEvent" prim_putBackEvent :: Addr -> Addr -> IO ()
peekEvent :: Display -> XEventPtr -> IO ()
peekEvent gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_peekEvent arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_peekEvent" prim_peekEvent :: Addr -> Addr -> IO ()

-- XFilterEvent omitted (can't find documentation)
-- XIfEvent omitted (can't pass predicates (yet))
-- XCheckIfEvent omitted (can't pass predicates (yet))
-- XPeekIfEvent omitted (can't pass predicates (yet))

refreshKeyboardMapping :: XMappingEvent -> IO ()
refreshKeyboardMapping gc_arg1 =
  case gc_arg1 of { (gc_arg2,gc_arg3,gc_arg4) ->
  prim_refreshKeyboardMapping gc_arg2 gc_arg3 gc_arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_refreshKeyboardMapping" prim_refreshKeyboardMapping :: Int -> Char -> Int -> IO ()

-- XSynchronize omitted (returns C function)
-- XSetAfterFunction omitted (can't pass functions (yet))

----------------------------------------------------------------
-- Geometry
----------------------------------------------------------------

geometry :: Display -> Int -> String -> String -> Dimension -> Dimension -> Dimension -> Int -> Int -> IO (Int,Position,Position,Dimension,Dimension)
geometry gc_arg1 arg2 gc_arg2 gc_arg3 arg5 arg6 arg7 arg8 arg9 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  (marshall_string_ gc_arg3) >>= \ (arg4) ->
  prim_geometry arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9
  >>= \ gc_result ->
  access_prim_geometry_res1 gc_result >>= \ res1 ->
  access_prim_geometry_res2 gc_result >>= \ res2 ->
  access_prim_geometry_res3 gc_result >>= \ res3 ->
  access_prim_geometry_res4 gc_result >>= \ res4 ->
  access_prim_geometry_res5 gc_result >>= \ res5 ->
  let gc_res1 = ( intToInt32  (res2)) in
  let gc_res2 = ( intToInt32  (res3)) in
  (return ((res1,gc_res1,gc_res2,res4,res5)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_geometry" prim_geometry :: Addr -> Int -> Addr -> Addr -> Word32 -> Word32 -> Word32 -> Int -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_geometry_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_geometry_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_geometry_res3 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_geometry_res4 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_geometry_res5 :: Ptr a1 -> IO (Word32)

getGeometry :: Display -> Int -> Drawable -> IO (Window,Position,Position,Dimension,Dimension,Dimension,Int)
getGeometry gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_getGeometry arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_getGeometry_res1 gc_result >>= \ res1 ->
  access_prim_getGeometry_res2 gc_result >>= \ res2 ->
  access_prim_getGeometry_res3 gc_result >>= \ res3 ->
  access_prim_getGeometry_res4 gc_result >>= \ res4 ->
  access_prim_getGeometry_res5 gc_result >>= \ res5 ->
  access_prim_getGeometry_res6 gc_result >>= \ res6 ->
  access_prim_getGeometry_res7 gc_result >>= \ res7 ->
  access_prim_getGeometry_gc_failed gc_result >>= \ gc_failed ->
  access_prim_getGeometry_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else let gc_res1 = ( intToInt32  (res2)) in
       let gc_res2 = ( intToInt32  (res3)) in
       (return ((res1,gc_res1,gc_res2,res4,res5,res6,res7)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_getGeometry" prim_getGeometry :: Addr -> Int -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res3 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res4 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res6 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_res7 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getGeometry_gc_failstring :: Ptr a1 -> IO (Addr)

-- XParseGeometry omitted (returned bitset too weird)

----------------------------------------------------------------
-- Locale
----------------------------------------------------------------

supportsLocale :: IO Bool
supportsLocale =
  prim_supportsLocale
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_supportsLocale" prim_supportsLocale :: IO (Int)
setLocaleModifiers :: String -> IO String
setLocaleModifiers gc_arg1 =
  (marshall_string_ gc_arg1) >>= \ (arg1) ->
  prim_setLocaleModifiers arg1
  >>= \  res1  ->
  (unmarshall_string_ res1) >>= \ gc_res1 ->
  (return (gc_res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setLocaleModifiers" prim_setLocaleModifiers :: Addr -> IO (Addr)

----------------------------------------------------------------
-- Screen Saver
----------------------------------------------------------------

type AllowExposuresMode = Int
dontAllowExposures :: AllowExposuresMode
dontAllowExposures =
  unsafePerformIO(
    prim_dontAllowExposures
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_dontAllowExposures" prim_dontAllowExposures :: IO (Int)
allowExposures :: AllowExposuresMode
allowExposures =
  unsafePerformIO(
    prim_allowExposures
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allowExposures" prim_allowExposures :: IO (Int)
defaultExposures :: AllowExposuresMode
defaultExposures =
  unsafePerformIO(
    prim_defaultExposures
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultExposures" prim_defaultExposures :: IO (Int)

type PreferBlankingMode = Int
dontPreferBlanking :: PreferBlankingMode
dontPreferBlanking =
  unsafePerformIO(
    prim_dontPreferBlanking
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_dontPreferBlanking" prim_dontPreferBlanking :: IO (Int)
preferBlanking :: PreferBlankingMode
preferBlanking =
  unsafePerformIO(
    prim_preferBlanking
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_preferBlanking" prim_preferBlanking :: IO (Int)
defaultBlanking :: PreferBlankingMode
defaultBlanking =
  unsafePerformIO(
    prim_defaultBlanking
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultBlanking" prim_defaultBlanking :: IO (Int)

type ScreenSaverMode = Int
screenSaverActive :: ScreenSaverMode
screenSaverActive =
  unsafePerformIO(
    prim_screenSaverActive
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenSaverActive" prim_screenSaverActive :: IO (Int)
screenSaverReset :: ScreenSaverMode
screenSaverReset =
  unsafePerformIO(
    prim_screenSaverReset
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenSaverReset" prim_screenSaverReset :: IO (Int)

getScreenSaver :: Display -> IO (Int,Int,PreferBlankingMode,AllowExposuresMode)
getScreenSaver gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_getScreenSaver arg1
  >>= \ gc_result ->
  access_prim_getScreenSaver_res1 gc_result >>= \ res1 ->
  access_prim_getScreenSaver_res2 gc_result >>= \ res2 ->
  access_prim_getScreenSaver_res3 gc_result >>= \ res3 ->
  access_prim_getScreenSaver_res4 gc_result >>= \ res4 ->
  (return ((res1,res2,res3,res4)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_getScreenSaver" prim_getScreenSaver :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getScreenSaver_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getScreenSaver_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getScreenSaver_res3 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getScreenSaver_res4 :: Ptr a1 -> IO (Int)

setScreenSaver :: Display -> Int -> Int -> PreferBlankingMode -> AllowExposuresMode -> IO ()
setScreenSaver gc_arg1 arg2 arg3 arg4 arg5 =
  case gc_arg1 of { (Display arg1) ->
  prim_setScreenSaver arg1 arg2 arg3 arg4 arg5}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setScreenSaver" prim_setScreenSaver :: Addr -> Int -> Int -> Int -> Int -> IO ()
activateScreenSaver :: Display -> IO ()
activateScreenSaver gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_activateScreenSaver arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_activateScreenSaver" prim_activateScreenSaver :: Addr -> IO ()
resetScreenSaver :: Display -> IO ()
resetScreenSaver gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_resetScreenSaver arg1}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_resetScreenSaver" prim_resetScreenSaver :: Addr -> IO ()
forceScreenSaver :: Display -> ScreenSaverMode -> IO ()
forceScreenSaver gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_forceScreenSaver arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_forceScreenSaver" prim_forceScreenSaver :: Addr -> Int -> IO ()


----------------------------------------------------------------
-- Pointer
----------------------------------------------------------------

getPointerControl :: Display -> IO (Int,Int,Int)
getPointerControl gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_getPointerControl arg1
  >>= \ gc_result ->
  access_prim_getPointerControl_res1 gc_result >>= \ res1 ->
  access_prim_getPointerControl_res2 gc_result >>= \ res2 ->
  access_prim_getPointerControl_res3 gc_result >>= \ res3 ->
  (return ((res1,res2,res3)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_getPointerControl" prim_getPointerControl :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getPointerControl_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getPointerControl_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getPointerControl_res3 :: Ptr a1 -> IO (Int)

warpPointer :: Display -> Window -> Window -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> IO ()
warpPointer gc_arg1 arg2 arg3 gc_arg2 gc_arg3 arg6 arg7 gc_arg4 gc_arg5 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  case ( int32ToInt  gc_arg4) of { arg8 ->
  case ( int32ToInt  gc_arg5) of { arg9 ->
  prim_warpPointer arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_warpPointer" prim_warpPointer :: Addr -> Word32 -> Word32 -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> IO ()

-- XGetPointerMapping omitted
-- XSetPointerMapping omitted

----------------------------------------------------------------
-- Visuals
----------------------------------------------------------------

-- XVisualIDFromVisual omitted


----------------------------------------------------------------
-- Threads
----------------------------------------------------------------

-- XInitThreads omitted (leary of thread stuff)
-- XLockDisplay omitted (leary of thread stuff)
-- XUnlockDisplay omitted (leary of thread stuff)

----------------------------------------------------------------
-- Screen
----------------------------------------------------------------

-- Many flags assumed to be PURE.

blackPixelOfScreen :: Screen -> Pixel
blackPixelOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_blackPixelOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_blackPixelOfScreen" prim_blackPixelOfScreen :: Addr -> IO (Word32)
whitePixelOfScreen :: Screen -> Pixel
whitePixelOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_whitePixelOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_whitePixelOfScreen" prim_whitePixelOfScreen :: Addr -> IO (Word32)
cellsOfScreen :: Screen -> Int
cellsOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_cellsOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_cellsOfScreen" prim_cellsOfScreen :: Addr -> IO (Int)
defaultColormapOfScreen :: Screen -> Colormap
defaultColormapOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_defaultColormapOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultColormapOfScreen" prim_defaultColormapOfScreen :: Addr -> IO (Word32)
defaultDepthOfScreen :: Screen -> Int
defaultDepthOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_defaultDepthOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultDepthOfScreen" prim_defaultDepthOfScreen :: Addr -> IO (Int)
defaultGCOfScreen :: Screen -> GC
defaultGCOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_defaultGCOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultGCOfScreen" prim_defaultGCOfScreen :: Addr -> IO (Addr)
defaultVisualOfScreen :: Screen -> Visual
defaultVisualOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_defaultVisualOfScreen arg1
    >>= \  res1  ->
    (return ((Visual res1)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defaultVisualOfScreen" prim_defaultVisualOfScreen :: Addr -> IO (Addr)
doesBackingStore :: Screen -> Bool
doesBackingStore gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_doesBackingStore arg1
    >>= \  res1  ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_doesBackingStore" prim_doesBackingStore :: Addr -> IO (Int)
doesSaveUnders :: Screen -> Bool
doesSaveUnders gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_doesSaveUnders arg1
    >>= \  res1  ->
    (unmarshall_bool_ res1) >>= \ gc_res1 ->
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_doesSaveUnders" prim_doesSaveUnders :: Addr -> IO (Int)
displayOfScreen :: Screen -> Display
displayOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_displayOfScreen arg1
    >>= \  res1  ->
    (return ((Display res1)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayOfScreen" prim_displayOfScreen :: Addr -> IO (Addr)

-- event mask at connection setup time - not current event mask!
eventMaskOfScreen :: Screen -> EventMask
eventMaskOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_eventMaskOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_eventMaskOfScreen" prim_eventMaskOfScreen :: Addr -> IO (Word32)

minCmapsOfScreen :: Screen -> Int
minCmapsOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_minCmapsOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_minCmapsOfScreen" prim_minCmapsOfScreen :: Addr -> IO (Int)
maxCmapsOfScreen :: Screen -> Int
maxCmapsOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_maxCmapsOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_maxCmapsOfScreen" prim_maxCmapsOfScreen :: Addr -> IO (Int)
rootWindowOfScreen :: Screen -> Window
rootWindowOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_rootWindowOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rootWindowOfScreen" prim_rootWindowOfScreen :: Addr -> IO (Word32)
widthOfScreen :: Screen -> Dimension
widthOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_widthOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_widthOfScreen" prim_widthOfScreen :: Addr -> IO (Word32)
widthMMOfScreen :: Screen -> Dimension
widthMMOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_widthMMOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_widthMMOfScreen" prim_widthMMOfScreen :: Addr -> IO (Word32)
heightOfScreen :: Screen -> Dimension
heightOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_heightOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_heightOfScreen" prim_heightOfScreen :: Addr -> IO (Word32)
heightMMOfScreen :: Screen -> Dimension
heightMMOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_heightMMOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_heightMMOfScreen" prim_heightMMOfScreen :: Addr -> IO (Word32)
planesOfScreen :: Screen -> Int
planesOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_planesOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_planesOfScreen" prim_planesOfScreen :: Addr -> IO (Int)
screenNumberOfScreen :: Screen -> ScreenNumber
screenNumberOfScreen gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Screen arg1) ->
    prim_screenNumberOfScreen arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_screenNumberOfScreen" prim_screenNumberOfScreen :: Addr -> IO (Word32)


----------------------------------------------------------------
-- Graphics contexts
----------------------------------------------------------------

-- Convenience functions

setArcMode :: Display -> GC -> ArcMode -> IO ()
setArcMode gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setArcMode arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setArcMode" prim_setArcMode :: Addr -> Addr -> Int -> IO ()
setBackground :: Display -> GC -> Pixel -> IO ()
setBackground gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setBackground arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setBackground" prim_setBackground :: Addr -> Addr -> Word32 -> IO ()
setForeground :: Display -> GC -> Pixel -> IO ()
setForeground gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setForeground arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setForeground" prim_setForeground :: Addr -> Addr -> Word32 -> IO ()
setFunction :: Display -> GC -> GXFunction -> IO ()
setFunction gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setFunction arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setFunction" prim_setFunction :: Addr -> Addr -> Int -> IO ()
setGraphicsExposures :: Display -> GC -> Bool -> IO ()
setGraphicsExposures gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_bool_ gc_arg2) >>= \ (arg3) ->
  prim_setGraphicsExposures arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setGraphicsExposures" prim_setGraphicsExposures :: Addr -> Addr -> Int -> IO ()
setClipMask :: Display -> GC -> Pixmap -> IO ()
setClipMask gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setClipMask arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setClipMask" prim_setClipMask :: Addr -> Addr -> Word32 -> IO ()
setClipOrigin :: Display -> GC -> Position -> Position -> IO ()
setClipOrigin gc_arg1 arg2 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  prim_setClipOrigin arg1 arg2 arg3 arg4}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setClipOrigin" prim_setClipOrigin :: Addr -> Addr -> Int -> Int -> IO ()

-- XSetClipRectangles omitted because it's not clear when it's safe to delete the
-- array of rectangles

setDashes :: Display -> GC -> Int -> String -> Int -> IO ()
setDashes gc_arg1 arg2 arg3 gc_arg2 arg5 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg4) ->
  prim_setDashes arg1 arg2 arg3 arg4 arg5}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setDashes" prim_setDashes :: Addr -> Addr -> Int -> Addr -> Int -> IO ()
setFillRule :: Display -> GC -> FillRule -> IO ()
setFillRule gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setFillRule arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setFillRule" prim_setFillRule :: Addr -> Addr -> Int -> IO ()
setFillStyle :: Display -> GC -> FillStyle -> IO ()
setFillStyle gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setFillStyle arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setFillStyle" prim_setFillStyle :: Addr -> Addr -> Int -> IO ()
setFont :: Display -> GC -> Font -> IO ()
setFont gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setFont arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setFont" prim_setFont :: Addr -> Addr -> Word32 -> IO ()
setLineAttributes :: Display -> GC -> Int -> LineStyle -> CapStyle -> JoinStyle -> IO ()
setLineAttributes gc_arg1 arg2 arg3 arg4 arg5 arg6 =
  case gc_arg1 of { (Display arg1) ->
  prim_setLineAttributes arg1 arg2 arg3 arg4 arg5 arg6}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setLineAttributes" prim_setLineAttributes :: Addr -> Addr -> Int -> Int -> Int -> Int -> IO ()
setPlaneMask :: Display -> GC -> Pixel -> IO ()
setPlaneMask gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setPlaneMask arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setPlaneMask" prim_setPlaneMask :: Addr -> Addr -> Word32 -> IO ()
setState :: Display -> GC -> Pixel -> Pixel -> GXFunction -> Pixel -> IO ()
setState gc_arg1 arg2 arg3 arg4 arg5 arg6 =
  case gc_arg1 of { (Display arg1) ->
  prim_setState arg1 arg2 arg3 arg4 arg5 arg6}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setState" prim_setState :: Addr -> Addr -> Word32 -> Word32 -> Int -> Word32 -> IO ()
setStipple :: Display -> GC -> Pixmap -> IO ()
setStipple gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setStipple arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setStipple" prim_setStipple :: Addr -> Addr -> Word32 -> IO ()
setSubwindowMode :: Display -> GC -> SubWindowMode -> IO ()
setSubwindowMode gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setSubwindowMode arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setSubwindowMode" prim_setSubwindowMode :: Addr -> Addr -> Int -> IO ()
setTSOrigin :: Display -> GC -> Position -> Position -> IO ()
setTSOrigin gc_arg1 arg2 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  prim_setTSOrigin arg1 arg2 arg3 arg4}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setTSOrigin" prim_setTSOrigin :: Addr -> Addr -> Int -> Int -> IO ()
setTile :: Display -> GC -> Pixmap -> IO ()
setTile gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setTile arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setTile" prim_setTile :: Addr -> Addr -> Word32 -> IO ()

-- ToDo: create a real interface to this
createGC :: Display -> Drawable -> IO GC
createGC gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_createGC arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createGC" prim_createGC :: Addr -> Word32 -> IO (Addr)

-- OLD: 
-- %synonym : GCValueSet : Ptr
--   in rtsDummy
-- 
-- {%
-- typedef unsigned long GCMask; /* cf XtGCMask */
-- typedef struct _gcvalues {
--     GCMask	mask;
--     XGCValues	values;
-- }* GCValueSet;
-- %}
--
-- IMPURE GCValueSet	emptyGCValueSet()
-- RESULT: (RETVAL = (GCValueSet) malloc(sizeof(struct _gcvalues))) ? RETVAL->mask = 0, RETVAL : RETVAL;
-- POST: RETVAL != NULL
-- 
-- IMPURE void	setGCForeground(colour, set)
-- IN Pixel	colour
-- IN GCValueSet	set
-- RESULT: set->mask |= GCForeground; set->values.foreground = colour
-- 
-- IMPURE void	setGCBackground(colour, set)
-- IN Pixel	colour
-- IN GCValueSet	set
-- RESULT: set->mask |= GCBackground; set->values.background = colour
-- 
-- IMPURE void	freeGCValueSet(set)
-- IN GCValueSet	set
-- RESULT: free(set)
-- 
-- IMPURE GC	XCreateGC(display, d, set->mask, &(set->values))
-- NAME: xCreateGC
-- IN Display*	display
-- IN Drawable	d
-- IN GCValueSet	set
-- 
-- IMPURE void	XChangeGC(display, gc, set->mask, &(set->values))
-- NAME: xChangeGC
-- IN Display*	display
-- IN GC		gc
-- IN GCValueSet	set
-- 
-- STARTH
-- -- Code that packages GCValueSets up in a clean monoidic way.
-- 
-- data GCSetter = GCSetter (GCValueSet -> IO ()) -- should be newtype 
-- 
-- createGC :: Display -> Drawable -> GCSetter -> IO GC
-- createGC display d (GCSetter setter) =
--   emptyGCValueSet        >>= \ set ->
--   setter set             >>
--   xCreateGC display d set >>= \ gc ->
--   freeGCValueSet set     >>
--   return gc
-- 
-- changeGC :: Display -> Drawable -> GC -> GCSetter -> IO ()
-- changeGC display d gc (GCSetter setter) =
--   emptyGCValueSet        >>= \ set ->
--   setter set             >>
--   xChangeGC display d set >>= \ gc ->
--   freeGCValueSet set
-- 
-- instance Monoid GCSetter where
--   (GCSetter m) >>> (GCSetter k) 
--     = GCSetter (\ settings -> m settings >> k settings)
--   unit = GCSetter (\ _ -> return ())
-- 
-- set_Background :: Pixel -> GCSetter
-- set_Background c = GCSetter (setGCBackground c)
-- 
-- set_Foreground :: Pixel -> GCSetter
-- set_Foreground c = GCSetter (setGCForeground c)
-- ENDH

gContextFromGC :: GC -> GContext
gContextFromGC arg1 =
  unsafePerformIO(
    prim_gContextFromGC arg1
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_gContextFromGC" prim_gContextFromGC :: Addr -> IO (Word32)

freeGC :: Display -> GC -> IO ()
freeGC gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_freeGC arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freeGC" prim_freeGC :: Addr -> Addr -> IO ()
flushGC :: Display -> GC -> IO ()
flushGC gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_flushGC arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_flushGC" prim_flushGC :: Addr -> Addr -> IO ()

copyGC :: Display -> GC -> Mask -> GC -> IO ()
copyGC gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_copyGC arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_copyGC" prim_copyGC :: Addr -> Addr -> Word32 -> Addr -> IO ()

----------------------------------------------------------------
-- Pixmaps
----------------------------------------------------------------

createPixmap :: Display -> Drawable -> Dimension -> Dimension -> Int -> IO Pixmap
createPixmap gc_arg1 arg2 arg3 arg4 arg5 =
  case gc_arg1 of { (Display arg1) ->
  prim_createPixmap arg1 arg2 arg3 arg4 arg5
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createPixmap" prim_createPixmap :: Addr -> Word32 -> Word32 -> Word32 -> Int -> IO (Word32)
freePixmap :: Display -> Pixmap -> IO ()
freePixmap gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_freePixmap arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freePixmap" prim_freePixmap :: Addr -> Word32 -> IO ()

-- XCreatePixmapFromBitmapData omitted (type looks strange)

-- %fun XListPixmapFormatValues = res1 = XListPixmapFormatValues(display, &res1_size) :: Display -> ListXPixmapFormatValues

----------------------------------------------------------------
-- Bitmaps
----------------------------------------------------------------

-- ToDo: do these need to be available to the programmer?
-- Maybe I could just wire them into all other operations?

bitmapBitOrder :: Display -> ByteOrder
bitmapBitOrder gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_bitmapBitOrder arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_bitmapBitOrder" prim_bitmapBitOrder :: Addr -> IO (Int)
bitmapUnit :: Display -> Int
bitmapUnit gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_bitmapUnit arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_bitmapUnit" prim_bitmapUnit :: Addr -> IO (Int)
bitmapPad :: Display -> Int
bitmapPad gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_bitmapPad arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_bitmapPad" prim_bitmapPad :: Addr -> IO (Int)

-- ToDo: make sure that initialisation works correctly for x/y_hot
-- omitted
-- IMPURE void	XWriteBitmapFile(display, filename, bitmap, width, height, x_hot, y_hot) RAISES Either
-- RETURNTYPE	BitmapFileStatus
-- GLOBAL ERROR BitmapFileStatus	RETVAL
-- IN Display*	display
-- IN String	filename
-- IN Pixmap	bitmap
-- IN Dimension	width
-- IN Dimension	height
-- IN Maybe Int	x_hot = -1
-- IN Maybe Int	y_hot = -1
-- POST: RETVAL == BitmapSuccess

-- omitted
-- IMPURE void	XReadBitmapFile(display, d, filename, bitmap, width, height, x_hot, y_hot) RAISES Either
-- RETURNTYPE	BitmapFileStatus
-- GLOBAL ERROR BitmapFileStatus	RETVAL
-- IN Display*	display
-- IN Drawable	d
-- IN String	filename
-- OUT Pixmap	bitmap
-- OUT Dimension	width
-- OUT Dimension	height
-- OUT Int		x_hot RAISES Maybe IF x_hot == -1
-- OUT Int		y_hot RAISES Maybe IF x_hot == -1
-- POST: RETVAL == BitmapSuccess

-- XCreateBitmapFromData omitted (awkward looking type)
-- XReadBitmapFileData omitted (awkward looking type)

----------------------------------------------------------------
-- Color and Colormaps
----------------------------------------------------------------

lookupColor :: Display -> Colormap -> String -> IO (Color,Color)
lookupColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  prim_lookupColor arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_lookupColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_lookupColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_lookupColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_lookupColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_lookupColor_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_lookupColor_gc_res10 gc_result >>= \ gc_res10 ->
  access_prim_lookupColor_gc_res12 gc_result >>= \ gc_res12 ->
  access_prim_lookupColor_gc_res14 gc_result >>= \ gc_res14 ->
  access_prim_lookupColor_gc_res16 gc_result >>= \ gc_res16 ->
  access_prim_lookupColor_gc_res18 gc_result >>= \ gc_res18 ->
  access_prim_lookupColor_gc_failed gc_result >>= \ gc_failed ->
  access_prim_lookupColor_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else let gc_res2 = ( word32ToWord16  (gc_res3)) in
       let gc_res4 = ( word32ToWord16  (gc_res5)) in
       let gc_res6 = ( word32ToWord16  (gc_res7)) in
       let gc_res8 = ( word32ToWord8   (gc_res9)) in
       let gc_res11 = ( word32ToWord16  (gc_res12)) in
       let gc_res13 = ( word32ToWord16  (gc_res14)) in
       let gc_res15 = ( word32ToWord16  (gc_res16)) in
       let gc_res17 = ( word32ToWord8   (gc_res18)) in
       (return (((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8),(gc_res10,gc_res11,gc_res13,gc_res15,gc_res17))))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_lookupColor" prim_lookupColor :: Addr -> Word32 -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res9 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res10 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res12 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res14 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res16 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_res18 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupColor_gc_failstring :: Ptr a1 -> IO (Addr)

allocNamedColor :: Display -> Colormap -> String -> IO (Color,Color)
allocNamedColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  prim_allocNamedColor arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_allocNamedColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_allocNamedColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_allocNamedColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_allocNamedColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_allocNamedColor_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_allocNamedColor_gc_res10 gc_result >>= \ gc_res10 ->
  access_prim_allocNamedColor_gc_res12 gc_result >>= \ gc_res12 ->
  access_prim_allocNamedColor_gc_res14 gc_result >>= \ gc_res14 ->
  access_prim_allocNamedColor_gc_res16 gc_result >>= \ gc_res16 ->
  access_prim_allocNamedColor_gc_res18 gc_result >>= \ gc_res18 ->
  access_prim_allocNamedColor_gc_failed gc_result >>= \ gc_failed ->
  access_prim_allocNamedColor_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else let gc_res2 = ( word32ToWord16  (gc_res3)) in
       let gc_res4 = ( word32ToWord16  (gc_res5)) in
       let gc_res6 = ( word32ToWord16  (gc_res7)) in
       let gc_res8 = ( word32ToWord8   (gc_res9)) in
       let gc_res11 = ( word32ToWord16  (gc_res12)) in
       let gc_res13 = ( word32ToWord16  (gc_res14)) in
       let gc_res15 = ( word32ToWord16  (gc_res16)) in
       let gc_res17 = ( word32ToWord8   (gc_res18)) in
       (return (((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8),(gc_res10,gc_res11,gc_res13,gc_res15,gc_res17))))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocNamedColor" prim_allocNamedColor :: Addr -> Word32 -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res9 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res10 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res12 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res14 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res16 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_res18 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocNamedColor_gc_failstring :: Ptr a1 -> IO (Addr)

allocColor :: Display -> Colormap -> Color -> IO Color
allocColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  prim_allocColor arg1 arg2 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11
  >>= \ gc_result ->
  access_prim_allocColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_allocColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_allocColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_allocColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_allocColor_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_allocColor_gc_failed gc_result >>= \ gc_failed ->
  access_prim_allocColor_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else let gc_res2 = ( word32ToWord16  (gc_res3)) in
       let gc_res4 = ( word32ToWord16  (gc_res5)) in
       let gc_res6 = ( word32ToWord16  (gc_res7)) in
       let gc_res8 = ( word32ToWord8   (gc_res9)) in
       (return ((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8)))}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocColor" prim_allocColor :: Addr -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_res9 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_allocColor_gc_failstring :: Ptr a1 -> IO (Addr)

parseColor :: Display -> Colormap -> String -> IO Color
parseColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  prim_parseColor arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_parseColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_parseColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_parseColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_parseColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_parseColor_gc_res9 gc_result >>= \ gc_res9 ->
  access_prim_parseColor_gc_failed gc_result >>= \ gc_failed ->
  access_prim_parseColor_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else let gc_res2 = ( word32ToWord16  (gc_res3)) in
       let gc_res4 = ( word32ToWord16  (gc_res5)) in
       let gc_res6 = ( word32ToWord16  (gc_res7)) in
       let gc_res8 = ( word32ToWord8   (gc_res9)) in
       (return ((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_parseColor" prim_parseColor :: Addr -> Word32 -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_res9 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_parseColor_gc_failstring :: Ptr a1 -> IO (Addr)

-- ToDo: Can't express relationship between arg4 and res1 properly (or arg5, res2)
-- %errfun Zero XAllocColorCells :: Display -> Colormap -> Bool -> Int -> Int -> IO (ListPixel, ListPixel) using err = XAllocColorCells(arg1,arg2,arg3,arg4_size,res1,arg5_size,res2) 

-- ToDo: Can't express relationship between arg4 and res1 properly
-- %errfun Zero XAllocColorPlanes :: Display -> Colormap -> Bool -> Int -> Int -> Int -> Int IO (ListPixel, Pixel, Pixel, Pixel) using err = XAllocColorPlanes(...)

freeColors :: Display -> Colormap -> ListPixel -> Pixel -> IO ()
freeColors gc_arg1 arg2 gc_arg2 arg4 =
  case gc_arg1 of { (Display arg1) ->
  (marshallWord32List gc_arg2) >>= \ (arg3,arg3_size) ->
  prim_freeColors arg1 arg2 arg3 arg3_size arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freeColors" prim_freeColors :: Addr -> Word32 -> Addr -> Int -> Word32 -> IO ()

storeColor :: Display -> Colormap -> Color -> IO ()
storeColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  prim_storeColor arg1 arg2 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_storeColor" prim_storeColor :: Addr -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO ()

-- %fun XStoreColors :: Display -> Colormap -> ListColor -> IO ()         
-- %code XStoreColors(arg1,arg2,arg3,arg3_size) 
-- %fun XStoreNamedColor :: Display -> Colormap -> String -> Pixel -> PrimaryMask -> IO ()

queryColor :: Display -> Colormap -> Color -> IO Color
queryColor gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  prim_queryColor arg1 arg2 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11
  >>= \ gc_result ->
  access_prim_queryColor_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_queryColor_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_queryColor_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_queryColor_gc_res7 gc_result >>= \ gc_res7 ->
  access_prim_queryColor_gc_res9 gc_result >>= \ gc_res9 ->
  let gc_res2 = ( word32ToWord16  (gc_res3)) in
  let gc_res4 = ( word32ToWord16  (gc_res5)) in
  let gc_res6 = ( word32ToWord16  (gc_res7)) in
  let gc_res8 = ( word32ToWord8   (gc_res9)) in
  (return ((gc_res1,gc_res2,gc_res4,gc_res6,gc_res8)))}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryColor" prim_queryColor :: Addr -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColor_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColor_gc_res3 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColor_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColor_gc_res7 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColor_gc_res9 :: Ptr a1 -> IO (Word32)

queryColors :: Display -> Colormap -> ListColor -> IO ListColor
queryColors gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocColors writeColor gc_arg2) >>= \ (arg3,arg3_size) ->
  prim_queryColors arg1 arg2 arg3 arg3_size
  >>= \ gc_result ->
  access_prim_queryColors_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_queryColors_gc_res3 gc_result >>= \ gc_res3 ->
  (unmarshallAddrList readColor (gc_res2,gc_res3)) >>= \ gc_res1 ->
  (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryColors" prim_queryColors :: Addr -> Word32 -> Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColors_gc_res2 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_queryColors_gc_res3 :: Ptr a1 -> IO (Int)

installColormap :: Display -> Colormap -> IO ()
installColormap gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_installColormap arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_installColormap" prim_installColormap :: Addr -> Word32 -> IO ()
uninstallColormap :: Display -> Colormap -> IO ()
uninstallColormap gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_uninstallColormap arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_uninstallColormap" prim_uninstallColormap :: Addr -> Word32 -> IO ()
copyColormapAndFree :: Display -> Colormap -> IO Colormap
copyColormapAndFree gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_copyColormapAndFree arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_copyColormapAndFree" prim_copyColormapAndFree :: Addr -> Word32 -> IO (Word32)
createColormap :: Display -> Window -> Visual -> ColormapAlloc -> IO Colormap
createColormap gc_arg1 arg2 gc_arg2 arg4 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (Visual arg3) ->
  prim_createColormap arg1 arg2 arg3 arg4
  >>= \  res1  ->
  (return (res1))}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createColormap" prim_createColormap :: Addr -> Word32 -> Addr -> Int -> IO (Word32)
freeColormap :: Display -> Colormap -> IO ()
freeColormap gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_freeColormap arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freeColormap" prim_freeColormap :: Addr -> Word32 -> IO ()


----------------------------------------------------------------
-- Fonts
----------------------------------------------------------------

-- A glyph (or Char2b) is a 16 bit character identification.
-- The top 8 bits are zero in many fonts.
type Glyph = Int

-- Disnae exist: %fun LoadFont       :: Display -> String -> IO Font
-- Disnae exist: %fun UnloadFont     :: Display -> Font -> IO ()

-- Argument can be a Font or a GContext.
-- But, if it's a GContext, the fontStruct will use the GContext as the 
-- FontID - which will cause most things to break so it's probably
-- safer using XGetGCValues to get a genuine font ID
queryFont :: Display -> Font -> IO FontStruct
queryFont gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_queryFont arg1 arg2
  >>= \  res1  ->
  (return ((FontStruct res1)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_queryFont" prim_queryFont :: Addr -> Word32 -> IO (Addr)

-- Note that this _WILL NOT WORK_ unless you have explicitly set the font.
-- I'm slowly but surely coming to the conclusion that Xlib is a pile of
-- steaming shit.
fontFromGC :: Display -> GC -> IO Font
fontFromGC gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_fontFromGC arg1 arg2
  >>= \ gc_result ->
  access_prim_fontFromGC_gc_res1 gc_result >>= \ gc_res1 ->
  access_prim_fontFromGC_gc_failed gc_result >>= \ gc_failed ->
  access_prim_fontFromGC_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fontFromGC" prim_fontFromGC :: Addr -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fontFromGC_gc_res1 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fontFromGC_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fontFromGC_gc_failstring :: Ptr a1 -> IO (Addr)

loadQueryFont :: Display -> String -> IO FontStruct
loadQueryFont gc_arg1 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg2) ->
  prim_loadQueryFont arg1 arg2
  >>= \ gc_result ->
  access_prim_loadQueryFont_res1 gc_result >>= \ res1 ->
  access_prim_loadQueryFont_gc_failed gc_result >>= \ gc_failed ->
  access_prim_loadQueryFont_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return ((FontStruct res1)))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_loadQueryFont" prim_loadQueryFont :: Addr -> Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_loadQueryFont_res1 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_loadQueryFont_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_loadQueryFont_gc_failstring :: Ptr a1 -> IO (Addr)

freeFont :: Display -> FontStruct -> IO ()
freeFont gc_arg1 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (FontStruct arg2) ->
  prim_freeFont arg1 arg2}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freeFont" prim_freeFont :: Addr -> Addr -> IO ()
-- %fun XSetFontPath  :: Display -> ListString  -> IO () using XSetFontPath(arg1,arg2,arg2_size)

fontFromFontStruct :: FontStruct -> Font
fontFromFontStruct gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (FontStruct arg1) ->
    prim_fontFromFontStruct arg1
    >>= \  res1  ->
    (return (res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fontFromFontStruct" prim_fontFromFontStruct :: Addr -> IO (Word32)
ascentFromFontStruct :: FontStruct -> Int32
ascentFromFontStruct gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (FontStruct arg1) ->
    prim_ascentFromFontStruct arg1
    >>= \  res1  ->
    let gc_res1 = ( intToInt32  (res1)) in
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_ascentFromFontStruct" prim_ascentFromFontStruct :: Addr -> IO (Int)
descentFromFontStruct :: FontStruct -> Int32
descentFromFontStruct gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (FontStruct arg1) ->
    prim_descentFromFontStruct arg1
    >>= \  res1  ->
    let gc_res1 = ( intToInt32  (res1)) in
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_descentFromFontStruct" prim_descentFromFontStruct :: Addr -> IO (Int)

-- %prim XGetFontPath :: Display -> IO ListString
--Int r_size;
--String* r = XGetFontPath(arg1,&r_size);
-- %update(r);
--XFreeFontPath(r);
--return;

-- %prim XListFonts :: Display -> String -> Int -> IO ListString
--Int r_size;
--String *r = XListFonts(arg1,arg2,arg3,&r_size);
-- %update(r);
--XFreeFontNames(r);
--return;

-- XListFontsWithInfo omitted (no support for FontStruct yet)

-- XQueryTextExtents omitted (no support for CharStruct yet)
-- XQueryTextExtents16 omitted (no support for CharStruct yet)

-- We marshall this across right away because it's usually one-off info
type CharStruct =
 ( Int            -- lbearing (origin to left edge of raster)
 , Int            -- rbearing (origin to right edge of raster)
 , Int            -- width    (advance to next char's origin)
 , Int            -- ascent   (baseline to top edge of raster)
 , Int            -- descent  (baseline to bottom edge of raster)
-- attributes omitted
 )


-- No need to put ths in the IO monad - this info is essentially constant
textExtents :: FontStruct -> String -> (FontDirection,Int32,Int32,CharStruct)
textExtents gc_arg1 gc_arg2 =
  unsafePerformIO(
    case gc_arg1 of { (FontStruct f) ->
    (marshall_stringLen_ gc_arg2) >>= \ (s,s_len) ->
    prim_textExtents f s s_len
    >>= \ gc_result ->
    access_prim_textExtents_direction gc_result >>= \ direction ->
    access_prim_textExtents_font_ascent gc_result >>= \ font_ascent ->
    access_prim_textExtents_font_descent gc_result >>= \ font_descent ->
    access_prim_textExtents_gc_res3 gc_result >>= \ gc_res3 ->
    access_prim_textExtents_gc_res4 gc_result >>= \ gc_res4 ->
    access_prim_textExtents_gc_res5 gc_result >>= \ gc_res5 ->
    access_prim_textExtents_gc_res6 gc_result >>= \ gc_res6 ->
    access_prim_textExtents_gc_res7 gc_result >>= \ gc_res7 ->
    let gc_res1 = ( intToInt32  (font_ascent)) in
    let gc_res2 = ( intToInt32  (font_descent)) in
    (return ((direction,gc_res1,gc_res2,(gc_res3,gc_res4,gc_res5,gc_res6,gc_res7))))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_textExtents" prim_textExtents :: Addr -> Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_direction :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_font_ascent :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_font_descent :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_gc_res3 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_gc_res5 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_gc_res6 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_textExtents_gc_res7 :: Ptr a1 -> IO (Int)

-- No need to put ths in the IO monad - this info is essentially constant
textWidth :: FontStruct -> String -> Int32
textWidth gc_arg1 gc_arg2 =
  unsafePerformIO(
    case gc_arg1 of { (FontStruct f) ->
    (marshall_stringLen_ gc_arg2) >>= \ (s,s_len) ->
    prim_textWidth f s s_len
    >>= \  res1  ->
    let gc_res1 = ( intToInt32  (res1)) in
    (return (gc_res1))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_textWidth" prim_textWidth :: Addr -> Addr -> Int -> IO (Int)

-- XTextExtents16 omitted
-- XTextWidth16 omitted

-- XGetFontProperty omitted
-- XFreeFontInfo omitted
-- XFreeFontNames omitted

-- XCreateFontSet omitted (no documentation available)
-- XFreeFontSet omitted (no documentation available)
-- XFontsOfFontSet omitted (no documentation available)
-- XBaseFontNameListOfFontSet omitted (no documentation available)
-- XLocaleOfFontSet omitted (no documentation available)
-- XExtentsOfFontSet omitted (no documentation available)

-- XContextDependentDrawing omitted
-- XDirectionalDependentDrawing omitted
-- XContextualDrawing omitted

-- XmbTextEscapement omitted
-- XwcTextEscapement omitted
-- XmbTextExtents omitted
-- XwcTextExtents omitted
-- XmbTextPerCharExtents omitted
-- XwcTextPerCharExtents omitted
-- XmbDrawText omitted
-- XwcDrawText omitted
-- XmbDrawString omitted
-- XwcDrawString omitted
-- XmbDrawImageString omitted
-- XwcDrawImageString omitted

-- XOpenIM omitted
-- XCloseIM omitted
-- XGetIMValues omitted
-- XSetIMValues omitted
-- DisplayOfIM omitted
-- XLocaleOfIM omitted

-- XCreateIC omitted
-- XDestroyIC omitted
-- XSetICFocus omitted
-- XUnsetICFocus omitted
-- XwcResetIC omitted
-- XmbResetIC omitted
-- XSetICValues omitted
-- XGetICValues omitted
-- XIMOfIC omitted

-- XRegisterIMInstantiateCallback omitted
-- XUnregisterIMInstantiateCallback omitted

-- XInternalConnectionNumbers omitted
-- XProcessInternalConnection omitted
-- XAddConnectionWatch omitted
-- XRemoveConnectionWatch omitted

-- XmbLookupString omitted
-- XwcLookupString omitted

----------------------------------------------------------------
-- Atoms
----------------------------------------------------------------

-- AC, 1/9/2000: Added definition for XInternAtom
internAtom :: Display -> String -> Bool -> IO Atom
internAtom gc_arg1 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg2) ->
  (marshall_bool_ gc_arg3) >>= \ (arg3) ->
  prim_internAtom arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_internAtom" prim_internAtom :: Addr -> Addr -> Int -> IO (Word32)

-- XInternAtoms omitted
-- XGetAtomName omitted
-- XGetAtomNames omitted
-- XConvertSelection omitted
-- XListProperties omitted
-- XChangeProperty omitted
-- XDeleteProperty omitted

----------------------------------------------------------------
-- Keycodes
----------------------------------------------------------------

displayKeycodes :: Display -> (Int,Int)
displayKeycodes gc_arg1 =
  unsafePerformIO(
    case gc_arg1 of { (Display arg1) ->
    prim_displayKeycodes arg1
    >>= \ gc_result ->
    access_prim_displayKeycodes_res1 gc_result >>= \ res1 ->
    access_prim_displayKeycodes_res2 gc_result >>= \ res2 ->
    (return ((res1,res2)))})
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_displayKeycodes" prim_displayKeycodes :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_displayKeycodes_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_displayKeycodes_res2 :: Ptr a1 -> IO (Int)
lookupKeysym :: XKeyEventPtr -> Int -> IO KeySym
lookupKeysym arg1 arg2 =
  prim_lookupKeysym arg1 arg2
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_lookupKeysym" prim_lookupKeysym :: Addr -> Int -> IO (Word32)
keycodeToKeysym :: Display -> KeyCode -> Int -> IO KeySym
keycodeToKeysym gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_keycodeToKeysym arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_keycodeToKeysym" prim_keycodeToKeysym :: Addr -> Char -> Int -> IO (Word32)
keysymToKeycode :: Display -> KeySym -> IO KeyCode
keysymToKeycode gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_keysymToKeycode arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_keysymToKeycode" prim_keysymToKeycode :: Addr -> Word32 -> IO (Char)
keysymToString :: KeySym -> String
keysymToString arg1 =
  unsafePerformIO(
    prim_keysymToString arg1
    >>= \  res1  ->
    (unmarshall_string_ res1) >>= \ gc_res1 ->
    (return (gc_res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_keysymToString" prim_keysymToString :: Word32 -> IO (Addr)
stringToKeysym :: String -> KeySym
stringToKeysym gc_arg1 =
  unsafePerformIO(
    (marshall_string_ gc_arg1) >>= \ (arg1) ->
    prim_stringToKeysym arg1
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_stringToKeysym" prim_stringToKeysym :: Addr -> IO (Word32)

-- I should not need to do this: the argument to maybeT below _should_
-- be treated as a C expression.
noSymbol :: KeySym
noSymbol =
  unsafePerformIO(
    prim_noSymbol
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_noSymbol" prim_noSymbol :: IO (Word32)

-- I tried to allocate the buffer below on the stack, but that was not a
-- good idea since the unmarshalling takes place in another stack frame.
-- I could use malloc, but then it seems tricky to free _after_ unmarshalling.
-- (%end free(buf) does not work: frees buffer too early.)
-- Making the buffer static works, but is potentially not thread safe.
-- But: the graphics library does not use X in a threadsafe way anyway
-- (does not import/call XInitThreads) and GHC is single threaded as far as
-- the external world is concerned(?).
-- OK, go static for now. Introduce a mutex if needs be.
-- XLookupString cannot handle compose, it seems.
lookupString :: XKeyEventPtr -> IO (Maybe KeySym,String)
lookupString e =
  prim_lookupString e
  >>= \ gc_result ->
  access_prim_lookupString_ks gc_result >>= \ ks ->
  access_prim_lookupString_buf gc_result >>= \ buf ->
  access_prim_lookupString_n gc_result >>= \ n ->
  (if noSymbol == (ks)
   then return Nothing
   else (return ((Just ks)))) >>= \ gc_res1 ->
  (unmarshall_stringLen_ buf n) >>= \ gc_res2 ->
  (return ((gc_res1,gc_res2)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_lookupString" prim_lookupString :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupString_ks :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupString_buf :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_lookupString_n :: Ptr a1 -> IO (Int)

-- XQueryKeymap omitted
-- XRebindKeysym omitted
-- XDeleteModifiermapEntry omitted
-- XInsertModifiermapEntry omitted
-- XNewModifiermap omitted
-- XFreeModifiermap omitted
-- XSetModifierMapping omitted
-- XGetModifierMapping omitted
-- XGetKeyboardMapping omitted

----------------------------------------------------------------
-- Image
----------------------------------------------------------------

-- XCreateImage omitted
-- XInitImage omitted
-- XGetImage omitted
-- XPutImage omitted
-- XGetSubImage omitted

----------------------------------------------------------------
-- Icons
----------------------------------------------------------------

getIconName :: Display -> Window -> IO String
getIconName gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_getIconName arg1 arg2
  >>= \ gc_result ->
  access_prim_getIconName_res1 gc_result >>= \ res1 ->
  access_prim_getIconName_gc_failed gc_result >>= \ gc_failed ->
  access_prim_getIconName_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (unmarshall_string_ res1) >>= \ gc_res1 ->
       (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_getIconName" prim_getIconName :: Addr -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getIconName_res1 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getIconName_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_getIconName_gc_failstring :: Ptr a1 -> IO (Addr)

setIconName :: Display -> Window -> String -> IO ()
setIconName gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  prim_setIconName arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setIconName" prim_setIconName :: Addr -> Word32 -> Addr -> IO ()

----------------------------------------------------------------
-- Cursors
----------------------------------------------------------------

defineCursor :: Display -> Window -> Cursor -> IO ()
defineCursor gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_defineCursor arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_defineCursor" prim_defineCursor :: Addr -> Word32 -> Word32 -> IO ()
undefineCursor :: Display -> Window -> IO ()
undefineCursor gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_undefineCursor arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_undefineCursor" prim_undefineCursor :: Addr -> Word32 -> IO ()

createPixmapCursor :: Display -> Pixmap -> Pixmap -> Color -> Color -> Dimension -> Dimension -> IO Cursor
createPixmapCursor gc_arg1 arg2 arg3 gc_arg2 gc_arg13 arg6 arg7 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  case gc_arg13 of { (gc_arg14,gc_arg15,gc_arg17,gc_arg19,gc_arg21) ->
  case ( word16ToWord32  gc_arg15) of { gc_arg16 ->
  case ( word16ToWord32  gc_arg17) of { gc_arg18 ->
  case ( word16ToWord32  gc_arg19) of { gc_arg20 ->
  case ( word8ToWord32   gc_arg21) of { gc_arg22 ->
  prim_createPixmapCursor arg1 arg2 arg3 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11 gc_arg14 gc_arg16 gc_arg18 gc_arg20 gc_arg22 arg6 arg7
  >>= \  res1  ->
  (return (res1))}}}}}}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createPixmapCursor" prim_createPixmapCursor :: Addr -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO (Word32)

createGlyphCursor :: Display -> Font -> Font -> Glyph -> Glyph -> Color -> Color -> IO Cursor
createGlyphCursor gc_arg1 arg2 arg3 arg4 arg5 gc_arg2 gc_arg13 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  case gc_arg13 of { (gc_arg14,gc_arg15,gc_arg17,gc_arg19,gc_arg21) ->
  case ( word16ToWord32  gc_arg15) of { gc_arg16 ->
  case ( word16ToWord32  gc_arg17) of { gc_arg18 ->
  case ( word16ToWord32  gc_arg19) of { gc_arg20 ->
  case ( word8ToWord32   gc_arg21) of { gc_arg22 ->
  prim_createGlyphCursor arg1 arg2 arg3 arg4 arg5 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11 gc_arg14 gc_arg16 gc_arg18 gc_arg20 gc_arg22
  >>= \  res1  ->
  (return (res1))}}}}}}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createGlyphCursor" prim_createGlyphCursor :: Addr -> Word32 -> Word32 -> Int -> Int -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO (Word32)

createFontCursor :: Display -> Glyph -> IO Cursor
createFontCursor gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_createFontCursor arg1 arg2
  >>= \  res1  ->
  (return (res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createFontCursor" prim_createFontCursor :: Addr -> Int -> IO (Word32)
freeCursor :: Display -> Font -> IO ()
freeCursor gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_freeCursor arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_freeCursor" prim_freeCursor :: Addr -> Word32 -> IO ()
recolorCursor :: Display -> Cursor -> Color -> Color -> IO ()
recolorCursor gc_arg1 arg2 gc_arg2 gc_arg13 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg4,gc_arg6,gc_arg8,gc_arg10) ->
  case ( word16ToWord32  gc_arg4) of { gc_arg5 ->
  case ( word16ToWord32  gc_arg6) of { gc_arg7 ->
  case ( word16ToWord32  gc_arg8) of { gc_arg9 ->
  case ( word8ToWord32   gc_arg10) of { gc_arg11 ->
  case gc_arg13 of { (gc_arg14,gc_arg15,gc_arg17,gc_arg19,gc_arg21) ->
  case ( word16ToWord32  gc_arg15) of { gc_arg16 ->
  case ( word16ToWord32  gc_arg17) of { gc_arg18 ->
  case ( word16ToWord32  gc_arg19) of { gc_arg20 ->
  case ( word8ToWord32   gc_arg21) of { gc_arg22 ->
  prim_recolorCursor arg1 arg2 gc_arg3 gc_arg5 gc_arg7 gc_arg9 gc_arg11 gc_arg14 gc_arg16 gc_arg18 gc_arg20 gc_arg22}}}}}}}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_recolorCursor" prim_recolorCursor :: Addr -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> Word32 -> IO ()

----------------------------------------------------------------
-- Window Manager stuff
----------------------------------------------------------------

-- XConfigureWMWindow omitted (can't find documentation)
-- XReconfigureWMWindow omitted (can't find documentation)
-- XWMGeometry omitted (can't find documentation)
-- XGetWMColormapWindows omitted (can't find documentation)
-- XSetWMColormapWindows omitted (can't find documentation)
-- XGetWMProtocols omitted

-- AC, 1/9/2000: Added definition for XSetWMProtocols
setWMProtocols :: Display -> Window -> ListAtom -> IO ()
setWMProtocols gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallWord32List gc_arg2) >>= \ (arg3,arg3_size) ->
  prim_setWMProtocols arg1 arg2 arg3 arg3_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWMProtocols" prim_setWMProtocols :: Addr -> Word32 -> Addr -> Int -> IO ()


----------------------------------------------------------------
-- Set Window Attributes
----------------------------------------------------------------

-- The following code is based on what you get from the %ptr declaration.
-- ToDo: generate this kind of stuff automatically.

type XSetWindowAttributesPtr = Addr
--  toXSetWindowAttributesPtr   :: Addr -> XSetWindowAttributesPtr,
--  fromXSetWindowAttributesPtr :: XSetWindowAttributesPtr -> Ptr

allocXSetWindowAttributes :: IO XSetWindowAttributesPtr
allocXSetWindowAttributes =
  prim_allocXSetWindowAttributes
  >>= \  res1  ->
  (return (res1))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_allocXSetWindowAttributes" prim_allocXSetWindowAttributes :: IO (Addr)


-- toXSetWindowAttributesPtr   x = x 
-- fromXSetWindowAttributesPtr x = x
-- 
-- instance Pointer XSetWindowAttributesPtr where
--   toPtr     = fromXSetWindowAttributesPtr
--   fromPtr   = toXSetWindowAttributesPtr
--   alloc     = malloc sizeofXSetWindowAttributes
--   plus p x  = fromPtr (plusPtr (toPtr p) (x * sizeofXSetWindowAttributes))
--   minus p q = (toPtr p `minusPtr` toPtr q) `div` sizeofXSetWindowAttributes
-- %}}
-- 
-- %type : XSetWindowAttributesPtr : XSetWindowAttributes* arg1 : %pack(Ptr,arg1);% : %unpack(Ptr,arg1,arg2);%
-- 
-- %const Int sizeofXSetWindowAttributes = sizeof(XSetWindowAttributes)

---------------- Access to individual fields ----------------

set_background_pixmap :: XSetWindowAttributesPtr -> Pixmap -> IO ()
set_background_pixmap arg1 arg2 =
  prim_set_background_pixmap arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_background_pixmap" prim_set_background_pixmap :: Addr -> Word32 -> IO ()

set_background_pixel :: XSetWindowAttributesPtr -> Pixel -> IO ()
set_background_pixel arg1 arg2 =
  prim_set_background_pixel arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_background_pixel" prim_set_background_pixel :: Addr -> Word32 -> IO ()

set_border_pixmap :: XSetWindowAttributesPtr -> Pixmap -> IO ()
set_border_pixmap arg1 arg2 =
  prim_set_border_pixmap arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_border_pixmap" prim_set_border_pixmap :: Addr -> Word32 -> IO ()

set_border_pixel :: XSetWindowAttributesPtr -> Pixel -> IO ()
set_border_pixel arg1 arg2 =
  prim_set_border_pixel arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_border_pixel" prim_set_border_pixel :: Addr -> Word32 -> IO ()

set_bit_gravity :: XSetWindowAttributesPtr -> BitGravity -> IO ()
set_bit_gravity arg1 arg2 =
  prim_set_bit_gravity arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_bit_gravity" prim_set_bit_gravity :: Addr -> Int -> IO ()

set_win_gravity :: XSetWindowAttributesPtr -> WindowGravity -> IO ()
set_win_gravity arg1 arg2 =
  prim_set_win_gravity arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_win_gravity" prim_set_win_gravity :: Addr -> Int -> IO ()

set_backing_store :: XSetWindowAttributesPtr -> BackingStore -> IO ()
set_backing_store arg1 arg2 =
  prim_set_backing_store arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_backing_store" prim_set_backing_store :: Addr -> Int -> IO ()

set_backing_planes :: XSetWindowAttributesPtr -> Pixel -> IO ()
set_backing_planes arg1 arg2 =
  prim_set_backing_planes arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_backing_planes" prim_set_backing_planes :: Addr -> Word32 -> IO ()

set_backing_pixel :: XSetWindowAttributesPtr -> Pixel -> IO ()
set_backing_pixel arg1 arg2 =
  prim_set_backing_pixel arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_backing_pixel" prim_set_backing_pixel :: Addr -> Word32 -> IO ()

set_save_under :: XSetWindowAttributesPtr -> Bool -> IO ()
set_save_under arg1 gc_arg1 =
  (marshall_bool_ gc_arg1) >>= \ (arg2) ->
  prim_set_save_under arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_save_under" prim_set_save_under :: Addr -> Int -> IO ()

set_event_mask :: XSetWindowAttributesPtr -> EventMask -> IO ()
set_event_mask arg1 arg2 =
  prim_set_event_mask arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_event_mask" prim_set_event_mask :: Addr -> Word32 -> IO ()

set_do_not_propagate_mask :: XSetWindowAttributesPtr -> EventMask -> IO ()
set_do_not_propagate_mask arg1 arg2 =
  prim_set_do_not_propagate_mask arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_do_not_propagate_mask" prim_set_do_not_propagate_mask :: Addr -> Word32 -> IO ()

set_override_redirect :: XSetWindowAttributesPtr -> Bool -> IO ()
set_override_redirect arg1 gc_arg1 =
  (marshall_bool_ gc_arg1) >>= \ (arg2) ->
  prim_set_override_redirect arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_override_redirect" prim_set_override_redirect :: Addr -> Int -> IO ()

set_colormap :: XSetWindowAttributesPtr -> Colormap -> IO ()
set_colormap arg1 arg2 =
  prim_set_colormap arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_colormap" prim_set_colormap :: Addr -> Word32 -> IO ()

set_cursor :: XSetWindowAttributesPtr -> Cursor -> IO ()
set_cursor arg1 arg2 =
  prim_set_cursor arg1 arg2
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_set_cursor" prim_set_cursor :: Addr -> Word32 -> IO ()

----------------------------------------------------------------
-- Windows
----------------------------------------------------------------

storeName :: Display -> Window -> String -> IO ()
storeName gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_string_ gc_arg2) >>= \ (arg3) ->
  prim_storeName arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_storeName" prim_storeName :: Addr -> Word32 -> Addr -> IO ()
createSimpleWindow :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Int -> Pixel -> Pixel -> IO Window
createSimpleWindow gc_arg1 arg2 gc_arg2 gc_arg3 arg5 arg6 arg7 arg8 arg9 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  prim_createSimpleWindow arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9
  >>= \  res1  ->
  (return (res1))}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createSimpleWindow" prim_createSimpleWindow :: Addr -> Word32 -> Int -> Int -> Word32 -> Word32 -> Int -> Word32 -> Word32 -> IO (Word32)

createWindow :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> WindowClass -> Visual -> AttributeMask -> XSetWindowAttributesPtr -> IO Window
createWindow gc_arg1 arg2 gc_arg2 gc_arg3 arg5 arg6 arg7 arg8 arg9 gc_arg4 arg11 arg12 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  case gc_arg4 of { (Visual arg10) ->
  prim_createWindow arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10 arg11 arg12
  >>= \  res1  ->
  (return (res1))}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createWindow" prim_createWindow :: Addr -> Word32 -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> Int -> Addr -> Word32 -> Addr -> IO (Word32)

----------------------------------------------------------------

--ToDo: find an effective way to use Maybes
translateCoordinates :: Display -> Window -> Window -> Position -> Position -> IO (Bool,Position,Position,Window)
translateCoordinates gc_arg1 arg2 arg3 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_translateCoordinates arg1 arg2 arg3 arg4 arg5
  >>= \ gc_result ->
  access_prim_translateCoordinates_res1 gc_result >>= \ res1 ->
  access_prim_translateCoordinates_res2 gc_result >>= \ res2 ->
  access_prim_translateCoordinates_res3 gc_result >>= \ res3 ->
  access_prim_translateCoordinates_res4 gc_result >>= \ res4 ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  let gc_res2 = ( intToInt32  (res2)) in
  let gc_res3 = ( intToInt32  (res3)) in
  (return ((gc_res1,gc_res2,gc_res3,res4)))}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_translateCoordinates" prim_translateCoordinates :: Addr -> Word32 -> Word32 -> Int -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_translateCoordinates_res1 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_translateCoordinates_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_translateCoordinates_res3 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_translateCoordinates_res4 :: Ptr a1 -> IO (Word32)

moveResizeWindow :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> IO ()
moveResizeWindow gc_arg1 arg2 gc_arg2 gc_arg3 arg5 arg6 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  prim_moveResizeWindow arg1 arg2 arg3 arg4 arg5 arg6}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_moveResizeWindow" prim_moveResizeWindow :: Addr -> Word32 -> Int -> Int -> Word32 -> Word32 -> IO ()
resizeWindow :: Display -> Window -> Dimension -> Dimension -> IO ()
resizeWindow gc_arg1 arg2 arg3 arg4 =
  case gc_arg1 of { (Display arg1) ->
  prim_resizeWindow arg1 arg2 arg3 arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_resizeWindow" prim_resizeWindow :: Addr -> Word32 -> Word32 -> Word32 -> IO ()
moveWindow :: Display -> Window -> Position -> Position -> IO ()
moveWindow gc_arg1 arg2 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  prim_moveWindow arg1 arg2 arg3 arg4}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_moveWindow" prim_moveWindow :: Addr -> Word32 -> Int -> Int -> IO ()
reparentWindow :: Display -> Window -> Window -> Position -> Position -> IO ()
reparentWindow gc_arg1 arg2 arg3 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_reparentWindow arg1 arg2 arg3 arg4 arg5}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_reparentWindow" prim_reparentWindow :: Addr -> Word32 -> Word32 -> Int -> Int -> IO ()
mapSubwindows :: Display -> Window -> IO ()
mapSubwindows gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_mapSubwindows arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_mapSubwindows" prim_mapSubwindows :: Addr -> Word32 -> IO ()
unmapSubwindows :: Display -> Window -> IO ()
unmapSubwindows gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_unmapSubwindows arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_unmapSubwindows" prim_unmapSubwindows :: Addr -> Word32 -> IO ()
mapWindow :: Display -> Window -> IO ()
mapWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_mapWindow arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_mapWindow" prim_mapWindow :: Addr -> Word32 -> IO ()
-- Disnae exist: %fun XUnmapWindows                :: Display -> Window -> IO ()
-- Disnae exist: %fun XMapRaisedWindow             :: Display -> Window -> IO ()
lowerWindow :: Display -> Window -> IO ()
lowerWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_lowerWindow arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_lowerWindow" prim_lowerWindow :: Addr -> Word32 -> IO ()
raiseWindow :: Display -> Window -> IO ()
raiseWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_raiseWindow arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_raiseWindow" prim_raiseWindow :: Addr -> Word32 -> IO ()
circulateSubwindowsDown :: Display -> Window -> IO ()
circulateSubwindowsDown gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_circulateSubwindowsDown arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_circulateSubwindowsDown" prim_circulateSubwindowsDown :: Addr -> Word32 -> IO ()
circulateSubwindowsUp :: Display -> Window -> IO ()
circulateSubwindowsUp gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_circulateSubwindowsUp arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_circulateSubwindowsUp" prim_circulateSubwindowsUp :: Addr -> Word32 -> IO ()
circulateSubwindows :: Display -> Window -> CirculationDirection -> IO ()
circulateSubwindows gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_circulateSubwindows arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_circulateSubwindows" prim_circulateSubwindows :: Addr -> Word32 -> Int -> IO ()
iconifyWindow :: Display -> Window -> ScreenNumber -> IO ()
iconifyWindow gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_iconifyWindow arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_iconifyWindow_gc_failed gc_result >>= \ gc_failed ->
  access_prim_iconifyWindow_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_iconifyWindow" prim_iconifyWindow :: Addr -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_iconifyWindow_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_iconifyWindow_gc_failstring :: Ptr a1 -> IO (Addr)
withdrawWindow :: Display -> Window -> ScreenNumber -> IO ()
withdrawWindow gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_withdrawWindow arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_withdrawWindow_gc_failed gc_result >>= \ gc_failed ->
  access_prim_withdrawWindow_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_withdrawWindow" prim_withdrawWindow :: Addr -> Word32 -> Word32 -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_withdrawWindow_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_withdrawWindow_gc_failstring :: Ptr a1 -> IO (Addr)
destroyWindow :: Display -> Window -> IO ()
destroyWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_destroyWindow arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_destroyWindow" prim_destroyWindow :: Addr -> Word32 -> IO ()
destroySubwindows :: Display -> Window -> IO ()
destroySubwindows gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_destroySubwindows arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_destroySubwindows" prim_destroySubwindows :: Addr -> Word32 -> IO ()

setWindowBorder :: Display -> Window -> Pixel -> IO ()
setWindowBorder gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowBorder arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowBorder" prim_setWindowBorder :: Addr -> Word32 -> Word32 -> IO ()
setWindowBorderPixmap :: Display -> Window -> Pixmap -> IO ()
setWindowBorderPixmap gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowBorderPixmap arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowBorderPixmap" prim_setWindowBorderPixmap :: Addr -> Word32 -> Word32 -> IO ()
setWindowBorderWidth :: Display -> Window -> Dimension -> IO ()
setWindowBorderWidth gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowBorderWidth arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowBorderWidth" prim_setWindowBorderWidth :: Addr -> Word32 -> Word32 -> IO ()
setWindowBackground :: Display -> Window -> Pixel -> IO ()
setWindowBackground gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowBackground arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowBackground" prim_setWindowBackground :: Addr -> Word32 -> Word32 -> IO ()
setWindowBackgroundPixmap :: Display -> Window -> Pixmap -> IO ()
setWindowBackgroundPixmap gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowBackgroundPixmap arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowBackgroundPixmap" prim_setWindowBackgroundPixmap :: Addr -> Word32 -> Word32 -> IO ()
setWindowColormap :: Display -> Window -> Colormap -> IO ()
setWindowColormap gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_setWindowColormap arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setWindowColormap" prim_setWindowColormap :: Addr -> Word32 -> Word32 -> IO ()

addToSaveSet :: Display -> Window -> IO ()
addToSaveSet gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_addToSaveSet arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_addToSaveSet" prim_addToSaveSet :: Addr -> Word32 -> IO ()
removeFromSaveSet :: Display -> Window -> IO ()
removeFromSaveSet gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_removeFromSaveSet arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_removeFromSaveSet" prim_removeFromSaveSet :: Addr -> Word32 -> IO ()
changeSaveSet :: Display -> Window -> ChangeSaveSetMode -> IO ()
changeSaveSet gc_arg1 arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  prim_changeSaveSet arg1 arg2 arg3}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_changeSaveSet" prim_changeSaveSet :: Addr -> Word32 -> Int -> IO ()

clearWindow :: Display -> Window -> IO ()
clearWindow gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_clearWindow arg1 arg2}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_clearWindow" prim_clearWindow :: Addr -> Word32 -> IO ()
clearArea :: Display -> Window -> Position -> Position -> Dimension -> Dimension -> Bool -> IO ()
clearArea gc_arg1 arg2 gc_arg2 gc_arg3 arg5 arg6 gc_arg4 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg3 ->
  case ( int32ToInt  gc_arg3) of { arg4 ->
  (marshall_bool_ gc_arg4) >>= \ (arg7) ->
  prim_clearArea arg1 arg2 arg3 arg4 arg5 arg6 arg7}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_clearArea" prim_clearArea :: Addr -> Word32 -> Int -> Int -> Word32 -> Word32 -> Int -> IO ()

-- -- This is almost good enough - but doesn't call XFree
-- -- %errfun BadStatus XQueryTree :: Display -> Window -> IO (Window, Window, ListWindow) using err = XQueryTree(arg1,arg2,&res1,&res2,&res3,&res3_size)
-- %prim XQueryTree :: Display -> Window -> IO (Window, Window, ListWindow)
-- Window root_w, parent;
-- Int children_size;
-- Window *children;
-- Status r = XQueryTree(arg1,arg2,&root_w, &parent, &children, &children_size);
-- if (Success != r) { %failWith(BadStatus,r); }
-- %update(root_w,parent,children);
-- XFree(children);
-- return;

restackWindows :: Display -> ListWindow -> IO ()
restackWindows gc_arg1 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallWord32List gc_arg2) >>= \ (arg2,arg2_size) ->
  prim_restackWindows arg1 arg2 arg2_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_restackWindows" prim_restackWindows :: Addr -> Addr -> Int -> IO ()

-- -- ToDo: I want to be able to write this
-- -- %fun XListInstalledColormaps :: Display -> Window -> IO ListColormap using res1 = XListInstalledColormaps(arg1,arg2,&res1_size) 
-- -- But I have to write this instead - need to add a notion of cleanup code!
-- %prim XListInstalledColormaps :: Display -> Window -> IO ListColormap
-- Int r_size;
-- Colormap* r = XListInstalledColormaps(arg1,arg2,&r_size);
-- %update(r);
-- XFree(r);
-- return;
-- 
-- -- Again, this is almost good enough
-- -- %errfun BadStatus XGetCommand :: Display -> Window -> IO ListString using err = XGetCommand(arg1,arg2,&res1,&res1_size)
-- -- but not quite
-- -- %prim XGetCommand :: Display -> Window -> IO ListString
-- --Int    argv_size;
-- --String *argv;
-- --Status r = XGetCommand(arg1,arg2,&argv,&argv_size);
-- --if (Success != r) { %failWith(BadStatus, r); }
-- -- %update(argv);
-- --XFreeStringList(argv);
-- --return;
-- 
-- -- %fun XSetCommand :: Display -> Window -> ListString -> IO ()            using XSetCommand(arg1,arg2,arg3,res3_size)
-- 
-- %errfun BadStatus XGetTransientForHint :: Display -> Window -> IO Window using err = XGetTransientForHint(arg1,arg2,&res1)
-- 
-- %fun XSetTransientForHint :: Display -> Window -> Window -> IO ()
-- 
-- -- XRotateWindowProperties omitted
-- -- XGetWindowProperty omitted
-- 
-- -- XGetWindowAttributes omitted
-- -- XChangeWindowAttributes omitted

----------------------------------------------------------------
-- Drawing
----------------------------------------------------------------

drawPoint :: Display -> Drawable -> GC -> Position -> Position -> IO ()
drawPoint gc_arg1 arg2 arg3 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_drawPoint arg1 arg2 arg3 arg4 arg5}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawPoint" prim_drawPoint :: Addr -> Word32 -> Addr -> Int -> Int -> IO ()

drawPoints :: Display -> Drawable -> GC -> ListPoint -> CoordinateMode -> IO ()
drawPoints gc_arg1 arg2 arg3 gc_arg2 arg5 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocPoints writePoint gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_drawPoints arg1 arg2 arg3 arg4 arg4_size arg5}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawPoints" prim_drawPoints :: Addr -> Word32 -> Addr -> Addr -> Int -> Int -> IO ()

drawLine :: Display -> Drawable -> GC -> Position -> Position -> Position -> Position -> IO ()
drawLine gc_arg1 arg2 arg3 gc_arg2 gc_arg3 gc_arg4 gc_arg5 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  case ( int32ToInt  gc_arg4) of { arg6 ->
  case ( int32ToInt  gc_arg5) of { arg7 ->
  prim_drawLine arg1 arg2 arg3 arg4 arg5 arg6 arg7}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawLine" prim_drawLine :: Addr -> Word32 -> Addr -> Int -> Int -> Int -> Int -> IO ()

drawLines :: Display -> Drawable -> GC -> ListPoint -> CoordinateMode -> IO ()
drawLines gc_arg1 arg2 arg3 gc_arg2 arg5 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocPoints writePoint gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_drawLines arg1 arg2 arg3 arg4 arg4_size arg5}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawLines" prim_drawLines :: Addr -> Word32 -> Addr -> Addr -> Int -> Int -> IO ()

drawSegments :: Display -> Drawable -> GC -> ListSegment -> IO ()
drawSegments gc_arg1 arg2 arg3 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocSegments writeSegment gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_drawSegments arg1 arg2 arg3 arg4 arg4_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawSegments" prim_drawSegments :: Addr -> Word32 -> Addr -> Addr -> Int -> IO ()

drawRectangle :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> IO ()
drawRectangle gc_arg1 arg2 arg3 gc_arg2 gc_arg3 arg6 arg7 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_drawRectangle arg1 arg2 arg3 arg4 arg5 arg6 arg7}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawRectangle" prim_drawRectangle :: Addr -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> IO ()

drawRectangles :: Display -> Drawable -> GC -> ListRectangle -> IO ()
drawRectangles gc_arg1 arg2 arg3 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocRectangles writeRectangle gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_drawRectangles arg1 arg2 arg3 arg4 arg4_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawRectangles" prim_drawRectangles :: Addr -> Word32 -> Addr -> Addr -> Int -> IO ()

drawArc :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> IO ()
drawArc gc_arg1 arg2 arg3 gc_arg2 gc_arg3 arg6 arg7 arg8 arg9 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_drawArc arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawArc" prim_drawArc :: Addr -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> IO ()

drawArcs :: Display -> Drawable -> GC -> ListArc -> IO ()
drawArcs gc_arg1 arg2 arg3 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocArcs writeArc gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_drawArcs arg1 arg2 arg3 arg4 arg4_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawArcs" prim_drawArcs :: Addr -> Word32 -> Addr -> Addr -> Int -> IO ()

fillRectangle :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> IO ()
fillRectangle gc_arg1 arg2 arg3 gc_arg2 gc_arg3 arg6 arg7 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_fillRectangle arg1 arg2 arg3 arg4 arg5 arg6 arg7}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fillRectangle" prim_fillRectangle :: Addr -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> IO ()

fillRectangles :: Display -> Drawable -> GC -> ListRectangle -> IO ()
fillRectangles gc_arg1 arg2 arg3 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocRectangles writeRectangle gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_fillRectangles arg1 arg2 arg3 arg4 arg4_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fillRectangles" prim_fillRectangles :: Addr -> Word32 -> Addr -> Addr -> Int -> IO ()

fillPolygon :: Display -> Drawable -> GC -> ListPoint -> PolygonShape -> CoordinateMode -> IO ()
fillPolygon gc_arg1 arg2 arg3 gc_arg2 arg5 arg6 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocPoints writePoint gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_fillPolygon arg1 arg2 arg3 arg4 arg4_size arg5 arg6}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fillPolygon" prim_fillPolygon :: Addr -> Word32 -> Addr -> Addr -> Int -> Int -> Int -> IO ()

fillArc :: Display -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Int -> Int -> IO ()
fillArc gc_arg1 arg2 arg3 gc_arg2 gc_arg3 arg6 arg7 arg8 arg9 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  prim_fillArc arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fillArc" prim_fillArc :: Addr -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> IO ()

fillArcs :: Display -> Drawable -> GC -> ListArc -> IO ()
fillArcs gc_arg1 arg2 arg3 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshallAddrList allocArcs writeArc gc_arg2) >>= \ (arg4,arg4_size) ->
  prim_fillArcs arg1 arg2 arg3 arg4 arg4_size}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fillArcs" prim_fillArcs :: Addr -> Word32 -> Addr -> Addr -> Int -> IO ()

copyArea :: Display -> Drawable -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> IO ()
copyArea gc_arg1 arg2 arg3 arg4 gc_arg2 gc_arg3 arg7 arg8 gc_arg4 gc_arg5 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg5 ->
  case ( int32ToInt  gc_arg3) of { arg6 ->
  case ( int32ToInt  gc_arg4) of { arg9 ->
  case ( int32ToInt  gc_arg5) of { arg10 ->
  prim_copyArea arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_copyArea" prim_copyArea :: Addr -> Word32 -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> IO ()
copyPlane :: Display -> Drawable -> Drawable -> GC -> Position -> Position -> Dimension -> Dimension -> Position -> Position -> Pixel -> IO ()
copyPlane gc_arg1 arg2 arg3 arg4 gc_arg2 gc_arg3 arg7 arg8 gc_arg4 gc_arg5 arg11 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg5 ->
  case ( int32ToInt  gc_arg3) of { arg6 ->
  case ( int32ToInt  gc_arg4) of { arg9 ->
  case ( int32ToInt  gc_arg5) of { arg10 ->
  prim_copyPlane arg1 arg2 arg3 arg4 arg5 arg6 arg7 arg8 arg9 arg10 arg11}}}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_copyPlane" prim_copyPlane :: Addr -> Word32 -> Word32 -> Addr -> Int -> Int -> Word32 -> Word32 -> Int -> Int -> Word32 -> IO ()

-- draw characters over existing background
drawString :: Display -> Drawable -> GC -> Position -> Position -> String -> IO ()
drawString gc_arg1 arg2 arg3 gc_arg2 gc_arg3 gc_arg4 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  (marshall_stringLen_ gc_arg4) >>= \ (arg6,arg6_size) ->
  prim_drawString arg1 arg2 arg3 arg4 arg5 arg6 arg6_size}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawString" prim_drawString :: Addr -> Word32 -> Addr -> Int -> Int -> Addr -> Int -> IO ()

-- draw characters over a blank rectangle of current background colour
drawImageString :: Display -> Drawable -> GC -> Position -> Position -> String -> IO ()
drawImageString gc_arg1 arg2 arg3 gc_arg2 gc_arg3 gc_arg4 =
  case gc_arg1 of { (Display arg1) ->
  case ( int32ToInt  gc_arg2) of { arg4 ->
  case ( int32ToInt  gc_arg3) of { arg5 ->
  (marshall_stringLen_ gc_arg4) >>= \ (arg6,arg6_size) ->
  prim_drawImageString arg1 arg2 arg3 arg4 arg5 arg6 arg6_size}}}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_drawImageString" prim_drawImageString :: Addr -> Word32 -> Addr -> Int -> Int -> Addr -> Int -> IO ()

-- XDrawString16 omitted (16bit chars not supported)
-- XDrawImageString16 omitted (16bit chars not supported)
-- XDrawText omitted (XTextItem not supported)
-- XDrawText16 omitted (XTextItem not supported)

----------------------------------------------------------------
-- Cut and paste buffers
----------------------------------------------------------------

storeBuffer :: Display -> String -> Int -> IO ()
storeBuffer gc_arg1 gc_arg2 arg3 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_stringLen_ gc_arg2) >>= \ (arg2,arg2_size) ->
  prim_storeBuffer arg1 arg2 arg2_size arg3
  >>= \ gc_result ->
  access_prim_storeBuffer_gc_failed gc_result >>= \ gc_failed ->
  access_prim_storeBuffer_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_storeBuffer" prim_storeBuffer :: Addr -> Addr -> Int -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_storeBuffer_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_storeBuffer_gc_failstring :: Ptr a1 -> IO (Addr)

storeBytes :: Display -> String -> IO ()
storeBytes gc_arg1 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_stringLen_ gc_arg2) >>= \ (arg2,arg2_size) ->
  prim_storeBytes arg1 arg2 arg2_size
  >>= \ gc_result ->
  access_prim_storeBytes_gc_failed gc_result >>= \ gc_failed ->
  access_prim_storeBytes_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_storeBytes" prim_storeBytes :: Addr -> Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_storeBytes_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_storeBytes_gc_failstring :: Ptr a1 -> IO (Addr)

fetchBuffer :: Display -> Int -> IO String
fetchBuffer gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_fetchBuffer arg1 arg2
  >>= \ gc_result ->
  access_prim_fetchBuffer_res1 gc_result >>= \ res1 ->
  access_prim_fetchBuffer_nbytes gc_result >>= \ nbytes ->
  access_prim_fetchBuffer_gc_failed gc_result >>= \ gc_failed ->
  access_prim_fetchBuffer_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (unmarshall_stringLen_ res1 nbytes) >>= \ gc_res1 ->
       (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fetchBuffer" prim_fetchBuffer :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBuffer_res1 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBuffer_nbytes :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBuffer_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBuffer_gc_failstring :: Ptr a1 -> IO (Addr)

fetchBytes :: Display -> IO String
fetchBytes gc_arg1 =
  case gc_arg1 of { (Display arg1) ->
  prim_fetchBytes arg1
  >>= \ gc_result ->
  access_prim_fetchBytes_res1 gc_result >>= \ res1 ->
  access_prim_fetchBytes_nbytes gc_result >>= \ nbytes ->
  access_prim_fetchBytes_gc_failed gc_result >>= \ gc_failed ->
  access_prim_fetchBytes_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (unmarshall_stringLen_ res1 nbytes) >>= \ gc_res1 ->
       (return (gc_res1))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_fetchBytes" prim_fetchBytes :: Addr -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBytes_res1 :: Ptr a1 -> IO (Addr)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBytes_nbytes :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBytes_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_fetchBytes_gc_failstring :: Ptr a1 -> IO (Addr)

rotateBuffers :: Display -> Int -> IO ()
rotateBuffers gc_arg1 arg2 =
  case gc_arg1 of { (Display arg1) ->
  prim_rotateBuffers arg1 arg2
  >>= \ gc_result ->
  access_prim_rotateBuffers_gc_failed gc_result >>= \ gc_failed ->
  access_prim_rotateBuffers_gc_failstring gc_result >>= \ gc_failstring ->
  if ( gc_failed /= (0::Int))
  then unmarshall_string_ gc_failstring >>=  ioError  . userError
  else (return (()))}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rotateBuffers" prim_rotateBuffers :: Addr -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_rotateBuffers_gc_failed :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_rotateBuffers_gc_failstring :: Ptr a1 -> IO (Addr)

----------------------------------------------------------------
-- Atoms
----------------------------------------------------------------

a_PRIMARY :: Atom
a_PRIMARY =
  unsafePerformIO(
    prim_a_PRIMARY
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_PRIMARY" prim_a_PRIMARY :: IO (Word32)
a_SECONDARY :: Atom
a_SECONDARY =
  unsafePerformIO(
    prim_a_SECONDARY
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_SECONDARY" prim_a_SECONDARY :: IO (Word32)
a_ARC :: Atom
a_ARC =
  unsafePerformIO(
    prim_a_ARC
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_ARC" prim_a_ARC :: IO (Word32)
a_ATOM :: Atom
a_ATOM =
  unsafePerformIO(
    prim_a_ATOM
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_ATOM" prim_a_ATOM :: IO (Word32)
a_BITMAP :: Atom
a_BITMAP =
  unsafePerformIO(
    prim_a_BITMAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_BITMAP" prim_a_BITMAP :: IO (Word32)
a_CARDINAL :: Atom
a_CARDINAL =
  unsafePerformIO(
    prim_a_CARDINAL
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CARDINAL" prim_a_CARDINAL :: IO (Word32)
a_COLORMAP :: Atom
a_COLORMAP =
  unsafePerformIO(
    prim_a_COLORMAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_COLORMAP" prim_a_COLORMAP :: IO (Word32)
a_CURSOR :: Atom
a_CURSOR =
  unsafePerformIO(
    prim_a_CURSOR
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CURSOR" prim_a_CURSOR :: IO (Word32)
a_CUT_BUFFER0 :: Atom
a_CUT_BUFFER0 =
  unsafePerformIO(
    prim_a_CUT_BUFFER0
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER0" prim_a_CUT_BUFFER0 :: IO (Word32)
a_CUT_BUFFER1 :: Atom
a_CUT_BUFFER1 =
  unsafePerformIO(
    prim_a_CUT_BUFFER1
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER1" prim_a_CUT_BUFFER1 :: IO (Word32)
a_CUT_BUFFER2 :: Atom
a_CUT_BUFFER2 =
  unsafePerformIO(
    prim_a_CUT_BUFFER2
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER2" prim_a_CUT_BUFFER2 :: IO (Word32)
a_CUT_BUFFER3 :: Atom
a_CUT_BUFFER3 =
  unsafePerformIO(
    prim_a_CUT_BUFFER3
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER3" prim_a_CUT_BUFFER3 :: IO (Word32)
a_CUT_BUFFER4 :: Atom
a_CUT_BUFFER4 =
  unsafePerformIO(
    prim_a_CUT_BUFFER4
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER4" prim_a_CUT_BUFFER4 :: IO (Word32)
a_CUT_BUFFER5 :: Atom
a_CUT_BUFFER5 =
  unsafePerformIO(
    prim_a_CUT_BUFFER5
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER5" prim_a_CUT_BUFFER5 :: IO (Word32)
a_CUT_BUFFER6 :: Atom
a_CUT_BUFFER6 =
  unsafePerformIO(
    prim_a_CUT_BUFFER6
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER6" prim_a_CUT_BUFFER6 :: IO (Word32)
a_CUT_BUFFER7 :: Atom
a_CUT_BUFFER7 =
  unsafePerformIO(
    prim_a_CUT_BUFFER7
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CUT_BUFFER7" prim_a_CUT_BUFFER7 :: IO (Word32)
a_DRAWABLE :: Atom
a_DRAWABLE =
  unsafePerformIO(
    prim_a_DRAWABLE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_DRAWABLE" prim_a_DRAWABLE :: IO (Word32)
a_FONT :: Atom
a_FONT =
  unsafePerformIO(
    prim_a_FONT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_FONT" prim_a_FONT :: IO (Word32)
a_INTEGER :: Atom
a_INTEGER =
  unsafePerformIO(
    prim_a_INTEGER
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_INTEGER" prim_a_INTEGER :: IO (Word32)
a_PIXMAP :: Atom
a_PIXMAP =
  unsafePerformIO(
    prim_a_PIXMAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_PIXMAP" prim_a_PIXMAP :: IO (Word32)
a_POINT :: Atom
a_POINT =
  unsafePerformIO(
    prim_a_POINT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_POINT" prim_a_POINT :: IO (Word32)
a_RECTANGLE :: Atom
a_RECTANGLE =
  unsafePerformIO(
    prim_a_RECTANGLE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RECTANGLE" prim_a_RECTANGLE :: IO (Word32)
a_RESOURCE_MANAGER :: Atom
a_RESOURCE_MANAGER =
  unsafePerformIO(
    prim_a_RESOURCE_MANAGER
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RESOURCE_MANAGER" prim_a_RESOURCE_MANAGER :: IO (Word32)
a_RGB_COLOR_MAP :: Atom
a_RGB_COLOR_MAP =
  unsafePerformIO(
    prim_a_RGB_COLOR_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_COLOR_MAP" prim_a_RGB_COLOR_MAP :: IO (Word32)
a_RGB_BEST_MAP :: Atom
a_RGB_BEST_MAP =
  unsafePerformIO(
    prim_a_RGB_BEST_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_BEST_MAP" prim_a_RGB_BEST_MAP :: IO (Word32)
a_RGB_BLUE_MAP :: Atom
a_RGB_BLUE_MAP =
  unsafePerformIO(
    prim_a_RGB_BLUE_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_BLUE_MAP" prim_a_RGB_BLUE_MAP :: IO (Word32)
a_RGB_DEFAULT_MAP :: Atom
a_RGB_DEFAULT_MAP =
  unsafePerformIO(
    prim_a_RGB_DEFAULT_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_DEFAULT_MAP" prim_a_RGB_DEFAULT_MAP :: IO (Word32)
a_RGB_GRAY_MAP :: Atom
a_RGB_GRAY_MAP =
  unsafePerformIO(
    prim_a_RGB_GRAY_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_GRAY_MAP" prim_a_RGB_GRAY_MAP :: IO (Word32)
a_RGB_GREEN_MAP :: Atom
a_RGB_GREEN_MAP =
  unsafePerformIO(
    prim_a_RGB_GREEN_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_GREEN_MAP" prim_a_RGB_GREEN_MAP :: IO (Word32)
a_RGB_RED_MAP :: Atom
a_RGB_RED_MAP =
  unsafePerformIO(
    prim_a_RGB_RED_MAP
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RGB_RED_MAP" prim_a_RGB_RED_MAP :: IO (Word32)
a_STRING :: Atom
a_STRING =
  unsafePerformIO(
    prim_a_STRING
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_STRING" prim_a_STRING :: IO (Word32)
a_VISUALID :: Atom
a_VISUALID =
  unsafePerformIO(
    prim_a_VISUALID
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_VISUALID" prim_a_VISUALID :: IO (Word32)
a_WINDOW :: Atom
a_WINDOW =
  unsafePerformIO(
    prim_a_WINDOW
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WINDOW" prim_a_WINDOW :: IO (Word32)
a_WM_COMMAND :: Atom
a_WM_COMMAND =
  unsafePerformIO(
    prim_a_WM_COMMAND
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_COMMAND" prim_a_WM_COMMAND :: IO (Word32)
a_WM_HINTS :: Atom
a_WM_HINTS =
  unsafePerformIO(
    prim_a_WM_HINTS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_HINTS" prim_a_WM_HINTS :: IO (Word32)
a_WM_CLIENT_MACHINE :: Atom
a_WM_CLIENT_MACHINE =
  unsafePerformIO(
    prim_a_WM_CLIENT_MACHINE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_CLIENT_MACHINE" prim_a_WM_CLIENT_MACHINE :: IO (Word32)
a_WM_ICON_NAME :: Atom
a_WM_ICON_NAME =
  unsafePerformIO(
    prim_a_WM_ICON_NAME
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_ICON_NAME" prim_a_WM_ICON_NAME :: IO (Word32)
a_WM_ICON_SIZE :: Atom
a_WM_ICON_SIZE =
  unsafePerformIO(
    prim_a_WM_ICON_SIZE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_ICON_SIZE" prim_a_WM_ICON_SIZE :: IO (Word32)
a_WM_NAME :: Atom
a_WM_NAME =
  unsafePerformIO(
    prim_a_WM_NAME
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_NAME" prim_a_WM_NAME :: IO (Word32)
a_WM_NORMAL_HINTS :: Atom
a_WM_NORMAL_HINTS =
  unsafePerformIO(
    prim_a_WM_NORMAL_HINTS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_NORMAL_HINTS" prim_a_WM_NORMAL_HINTS :: IO (Word32)
a_WM_SIZE_HINTS :: Atom
a_WM_SIZE_HINTS =
  unsafePerformIO(
    prim_a_WM_SIZE_HINTS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_SIZE_HINTS" prim_a_WM_SIZE_HINTS :: IO (Word32)
a_WM_ZOOM_HINTS :: Atom
a_WM_ZOOM_HINTS =
  unsafePerformIO(
    prim_a_WM_ZOOM_HINTS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_ZOOM_HINTS" prim_a_WM_ZOOM_HINTS :: IO (Word32)
a_MIN_SPACE :: Atom
a_MIN_SPACE =
  unsafePerformIO(
    prim_a_MIN_SPACE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_MIN_SPACE" prim_a_MIN_SPACE :: IO (Word32)
a_NORM_SPACE :: Atom
a_NORM_SPACE =
  unsafePerformIO(
    prim_a_NORM_SPACE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_NORM_SPACE" prim_a_NORM_SPACE :: IO (Word32)
a_MAX_SPACE :: Atom
a_MAX_SPACE =
  unsafePerformIO(
    prim_a_MAX_SPACE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_MAX_SPACE" prim_a_MAX_SPACE :: IO (Word32)
a_END_SPACE :: Atom
a_END_SPACE =
  unsafePerformIO(
    prim_a_END_SPACE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_END_SPACE" prim_a_END_SPACE :: IO (Word32)
a_SUPERSCRIPT_X :: Atom
a_SUPERSCRIPT_X =
  unsafePerformIO(
    prim_a_SUPERSCRIPT_X
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_SUPERSCRIPT_X" prim_a_SUPERSCRIPT_X :: IO (Word32)
a_SUPERSCRIPT_Y :: Atom
a_SUPERSCRIPT_Y =
  unsafePerformIO(
    prim_a_SUPERSCRIPT_Y
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_SUPERSCRIPT_Y" prim_a_SUPERSCRIPT_Y :: IO (Word32)
a_SUBSCRIPT_X :: Atom
a_SUBSCRIPT_X =
  unsafePerformIO(
    prim_a_SUBSCRIPT_X
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_SUBSCRIPT_X" prim_a_SUBSCRIPT_X :: IO (Word32)
a_SUBSCRIPT_Y :: Atom
a_SUBSCRIPT_Y =
  unsafePerformIO(
    prim_a_SUBSCRIPT_Y
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_SUBSCRIPT_Y" prim_a_SUBSCRIPT_Y :: IO (Word32)
a_UNDERLINE_POSITION :: Atom
a_UNDERLINE_POSITION =
  unsafePerformIO(
    prim_a_UNDERLINE_POSITION
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_UNDERLINE_POSITION" prim_a_UNDERLINE_POSITION :: IO (Word32)
a_UNDERLINE_THICKNESS :: Atom
a_UNDERLINE_THICKNESS =
  unsafePerformIO(
    prim_a_UNDERLINE_THICKNESS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_UNDERLINE_THICKNESS" prim_a_UNDERLINE_THICKNESS :: IO (Word32)
a_STRIKEOUT_ASCENT :: Atom
a_STRIKEOUT_ASCENT =
  unsafePerformIO(
    prim_a_STRIKEOUT_ASCENT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_STRIKEOUT_ASCENT" prim_a_STRIKEOUT_ASCENT :: IO (Word32)
a_STRIKEOUT_DESCENT :: Atom
a_STRIKEOUT_DESCENT =
  unsafePerformIO(
    prim_a_STRIKEOUT_DESCENT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_STRIKEOUT_DESCENT" prim_a_STRIKEOUT_DESCENT :: IO (Word32)
a_ITALIC_ANGLE :: Atom
a_ITALIC_ANGLE =
  unsafePerformIO(
    prim_a_ITALIC_ANGLE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_ITALIC_ANGLE" prim_a_ITALIC_ANGLE :: IO (Word32)
a_X_HEIGHT :: Atom
a_X_HEIGHT =
  unsafePerformIO(
    prim_a_X_HEIGHT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_X_HEIGHT" prim_a_X_HEIGHT :: IO (Word32)
a_QUAD_WIDTH :: Atom
a_QUAD_WIDTH =
  unsafePerformIO(
    prim_a_QUAD_WIDTH
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_QUAD_WIDTH" prim_a_QUAD_WIDTH :: IO (Word32)
a_WEIGHT :: Atom
a_WEIGHT =
  unsafePerformIO(
    prim_a_WEIGHT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WEIGHT" prim_a_WEIGHT :: IO (Word32)
a_POINT_SIZE :: Atom
a_POINT_SIZE =
  unsafePerformIO(
    prim_a_POINT_SIZE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_POINT_SIZE" prim_a_POINT_SIZE :: IO (Word32)
a_RESOLUTION :: Atom
a_RESOLUTION =
  unsafePerformIO(
    prim_a_RESOLUTION
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_RESOLUTION" prim_a_RESOLUTION :: IO (Word32)
a_COPYRIGHT :: Atom
a_COPYRIGHT =
  unsafePerformIO(
    prim_a_COPYRIGHT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_COPYRIGHT" prim_a_COPYRIGHT :: IO (Word32)
a_NOTICE :: Atom
a_NOTICE =
  unsafePerformIO(
    prim_a_NOTICE
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_NOTICE" prim_a_NOTICE :: IO (Word32)
a_FONT_NAME :: Atom
a_FONT_NAME =
  unsafePerformIO(
    prim_a_FONT_NAME
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_FONT_NAME" prim_a_FONT_NAME :: IO (Word32)
a_FAMILY_NAME :: Atom
a_FAMILY_NAME =
  unsafePerformIO(
    prim_a_FAMILY_NAME
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_FAMILY_NAME" prim_a_FAMILY_NAME :: IO (Word32)
a_FULL_NAME :: Atom
a_FULL_NAME =
  unsafePerformIO(
    prim_a_FULL_NAME
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_FULL_NAME" prim_a_FULL_NAME :: IO (Word32)
a_CAP_HEIGHT :: Atom
a_CAP_HEIGHT =
  unsafePerformIO(
    prim_a_CAP_HEIGHT
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_CAP_HEIGHT" prim_a_CAP_HEIGHT :: IO (Word32)
a_WM_CLASS :: Atom
a_WM_CLASS =
  unsafePerformIO(
    prim_a_WM_CLASS
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_CLASS" prim_a_WM_CLASS :: IO (Word32)
a_WM_TRANSIENT_FOR :: Atom
a_WM_TRANSIENT_FOR =
  unsafePerformIO(
    prim_a_WM_TRANSIENT_FOR
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_WM_TRANSIENT_FOR" prim_a_WM_TRANSIENT_FOR :: IO (Word32)
a_LAST_PREDEFINED :: Atom
a_LAST_PREDEFINED =
  unsafePerformIO(
    prim_a_LAST_PREDEFINED
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_a_LAST_PREDEFINED" prim_a_LAST_PREDEFINED :: IO (Word32)

----------------------------------------------------------------
-- Window properties
----------------------------------------------------------------

setTextProperty :: Display -> Window -> String -> Atom -> IO ()
setTextProperty gc_arg1 arg2 gc_arg2 arg4 =
  case gc_arg1 of { (Display arg1) ->
  (marshall_stringLen_ gc_arg2) >>= \ (arg3,arg3_size) ->
  prim_setTextProperty arg1 arg2 arg3 arg3_size arg4}
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setTextProperty" prim_setTextProperty :: Addr -> Word32 -> Addr -> Int -> Word32 -> IO ()

-- %fun XSetStandardProperties :: Display -> Window -> String -> String -> Pixmap -> [String] -> XSizeHints -> IO ()
-- %code Status err = XSetStandardProperties(arg1,arg2,arg3,arg4,arg5,arg6,arg6_size,&arg7)
-- %fail { Success != err }{ BadStatus(err,XSetStandardProperties) }

----------------------------------------------------------------
-- Regions
----------------------------------------------------------------

type RectInRegionResult = Int

-- Return values from XRectInRegion()
rectangleOut :: RectInRegionResult
rectangleOut =
  unsafePerformIO(
    prim_rectangleOut
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rectangleOut" prim_rectangleOut :: IO (Int)
rectangleIn :: RectInRegionResult
rectangleIn =
  unsafePerformIO(
    prim_rectangleIn
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rectangleIn" prim_rectangleIn :: IO (Int)
rectanglePart :: RectInRegionResult
rectanglePart =
  unsafePerformIO(
    prim_rectanglePart
    >>= \  res1  ->
    (return (res1)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rectanglePart" prim_rectanglePart :: IO (Int)
 
-- regions deallocation is handled by the GC (ForeignObj magic)
-- so we don't provide XDestroyRegion explicitly
-- no idea what the int is for
-- %fun XDestroyRegion :: Region -> IO Int

----------------------------------------------------------------
-- Creating regions
----------------------------------------------------------------

-- an empty region 
-- (often used as "out argument" to binary operators which return regions)
createRegion :: IO Region
createRegion =
  prim_createRegion
  >>= \ gc_result ->
  access_prim_createRegion_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_createRegion_gc_res1 gc_result >>= \ gc_res1 ->
  makeForeignObj gc_res1 gc_res3 >>= \ gc_res2 ->
  (return ((Region gc_res2)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_createRegion" prim_createRegion :: IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_createRegion_gc_res3 :: Ptr a1 -> IO (Ptr ())
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_createRegion_gc_res1 :: Ptr a1 -> IO (Ptr ())

polygonRegion :: ListPoint -> FillRule -> IO Region
polygonRegion gc_arg1 arg3 =
  (marshallAddrList allocPoints writePoint gc_arg1) >>= \ (arg1,arg2) ->
  prim_polygonRegion arg1 arg2 arg3
  >>= \ gc_result ->
  access_prim_polygonRegion_gc_res3 gc_result >>= \ gc_res3 ->
  access_prim_polygonRegion_gc_res1 gc_result >>= \ gc_res1 ->
  (makeForeignObj gc_res1 gc_res3) >>= \ gc_res2 ->
  (return ((Region gc_res2)))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_polygonRegion" prim_polygonRegion :: Addr -> Int -> Int -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_polygonRegion_gc_res3 :: Ptr a1 -> IO (Ptr ())
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_polygonRegion_gc_res1 :: Ptr a1 -> IO (Ptr ())

----------------------------------------------------------------
-- Combining Regions
--
-- The usual shoddy state of Xlib documentation fails to mention
-- what the Int is for.
--
-- All operations overwrite the region in their third argument
-- which is usually a freshly created region.
----------------------------------------------------------------

intersectRegion :: Region -> Region -> Region -> IO Int
intersectRegion gc_arg1 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (Region arg2) ->
  case gc_arg3 of { (Region arg3) ->
  prim_intersectRegion arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}}
prim_intersectRegion :: ForeignObj -> ForeignObj -> ForeignObj -> IO (Int)
prim_intersectRegion a b c = 
		withForeignPtr a (\a' ->
		withForeignPtr b (\b' -> 
		withForeignPtr c (\c' -> prim_intersectRegion' a' b' c')))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_intersectRegion" prim_intersectRegion' :: ForeignObj' -> ForeignObj' -> ForeignObj' -> IO (Int)
subtractRegion :: Region -> Region -> Region -> IO Int
subtractRegion gc_arg1 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (Region arg2) ->
  case gc_arg3 of { (Region arg3) ->
  prim_subtractRegion arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}}
prim_subtractRegion :: ForeignObj -> ForeignObj -> ForeignObj -> IO (Int)
prim_subtractRegion a b c = 
		withForeignPtr a (\a' ->
		withForeignPtr b (\b' -> 
		withForeignPtr c (\c' -> prim_subtractRegion' a' b' c')))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_subtractRegion" prim_subtractRegion' :: ForeignObj' -> ForeignObj' -> ForeignObj' -> IO (Int)
unionRectWithRegion :: Rectangle -> Region -> Region -> IO Int
unionRectWithRegion gc_arg1 gc_arg9 gc_arg10 =
  case gc_arg1 of { (gc_arg2,gc_arg4,gc_arg6,gc_arg7) ->
  case ( int32ToInt  gc_arg2) of { gc_arg3 ->
  case ( int32ToInt  gc_arg4) of { gc_arg5 ->
  case gc_arg9 of { (Region arg2) ->
  case gc_arg10 of { (Region arg3) ->
  prim_unionRectWithRegion gc_arg3 gc_arg5 gc_arg6 gc_arg7 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}}}}
prim_unionRectWithRegion :: Int -> Int -> Word32 -> Word32 -> ForeignObj -> ForeignObj -> IO (Int)
prim_unionRectWithRegion a b c d e f = 
		withForeignPtr e (\e' ->
		withForeignPtr f (\f' -> prim_unionRectWithRegion' a b c d e' f'))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_unionRectWithRegion" prim_unionRectWithRegion' :: Int -> Int -> Word32 -> Word32 -> ForeignObj' -> ForeignObj' -> IO (Int)
unionRegion :: Region -> Region -> Region -> IO Int
unionRegion gc_arg1 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (Region arg2) ->
  case gc_arg3 of { (Region arg3) ->
  prim_unionRegion arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}}
prim_unionRegion :: ForeignObj -> ForeignObj -> ForeignObj -> IO (Int)
prim_unionRegion a b c = 
		withForeignPtr a (\a' ->
		withForeignPtr b (\b' -> 
		withForeignPtr c (\c' -> prim_unionRegion' a' b' c')))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_unionRegion" prim_unionRegion' :: ForeignObj' -> ForeignObj' -> ForeignObj' -> IO (Int)
xorRegion :: Region -> Region -> Region -> IO Int
xorRegion gc_arg1 gc_arg2 gc_arg3 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (Region arg2) ->
  case gc_arg3 of { (Region arg3) ->
  prim_xorRegion arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}}
prim_xorRegion :: ForeignObj -> ForeignObj -> ForeignObj -> IO (Int)
prim_xorRegion a b c =  
		withForeignPtr a (\a' ->
		withForeignPtr b (\b' -> 
		withForeignPtr c (\c' -> prim_xorRegion' a' b' c')))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_xorRegion" prim_xorRegion' :: ForeignObj' -> ForeignObj' -> ForeignObj' -> IO (Int)

----------------------------------------------------------------
-- Examining regions (tests, bounding boxes, etc)
----------------------------------------------------------------

emptyRegion :: Region -> IO Bool
emptyRegion gc_arg1 =
  case gc_arg1 of { (Region arg1) ->
  prim_emptyRegion arg1
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}
prim_emptyRegion :: ForeignObj -> IO (Int)
prim_emptyRegion a  = withForeignPtr a prim_emptyRegion'
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_emptyRegion" prim_emptyRegion' :: ForeignObj' -> IO (Int)
equalRegion :: Region -> Region -> IO Bool
equalRegion gc_arg1 gc_arg2 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (Region arg2) ->
  prim_equalRegion arg1 arg2
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}}
prim_equalRegion :: ForeignObj -> ForeignObj -> IO (Int)
prim_equalRegion a b = withForeignPtr a (\a' ->
		       withForeignPtr b (\b' -> prim_equalRegion' a' b'))
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_equalRegion" prim_equalRegion' :: ForeignObj' -> ForeignObj' -> IO (Int)
pointInRegion :: Region -> Point -> IO Bool
pointInRegion gc_arg1 gc_arg2 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg5) ->
  case ( int32ToInt  gc_arg3) of { gc_arg4 ->
  case ( int32ToInt  gc_arg5) of { gc_arg6 ->
  prim_pointInRegion arg1 gc_arg4 gc_arg6
  >>= \  res1  ->
  (unmarshall_bool_ res1) >>= \ gc_res1 ->
  (return (gc_res1))}}}}
prim_pointInRegion :: ForeignObj -> Int -> Int -> IO (Int)
prim_pointInRegion a b c = withForeignPtr a (\a' -> prim_pointInRegion' a' b c)
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_pointInRegion" prim_pointInRegion' :: ForeignObj' -> Int -> Int -> IO (Int)
rectInRegion :: Region -> Rectangle -> IO RectInRegionResult
rectInRegion gc_arg1 gc_arg2 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg5,gc_arg7,gc_arg8) ->
  case ( int32ToInt  gc_arg3) of { gc_arg4 ->
  case ( int32ToInt  gc_arg5) of { gc_arg6 ->
  prim_rectInRegion arg1 gc_arg4 gc_arg6 gc_arg7 gc_arg8
  >>= \  res1  ->
  (return (res1))}}}}
prim_rectInRegion :: ForeignObj -> Int -> Int -> Word32 -> Word32 -> IO (Int)
prim_rectInRegion a b c d e = withForeignPtr a (\a' -> 
						prim_rectInRegion' a' b c d e)
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_rectInRegion" prim_rectInRegion' :: ForeignObj' -> Int -> Int -> Word32 -> Word32 -> IO (Int)

-- I have no idea what the int is for
clipBox :: Region -> IO (Rectangle,Int)
clipBox gc_arg1 =
  case gc_arg1 of { (Region arg1) ->
  prim_clipBox arg1
  >>= \ gc_result ->
  access_prim_clipBox_gc_res2 gc_result >>= \ gc_res2 ->
  access_prim_clipBox_gc_res4 gc_result >>= \ gc_res4 ->
  access_prim_clipBox_gc_res5 gc_result >>= \ gc_res5 ->
  access_prim_clipBox_gc_res6 gc_result >>= \ gc_res6 ->
  access_prim_clipBox_res2 gc_result >>= \ res2 ->
  let gc_res1 = ( intToInt32  (gc_res2)) in
  let gc_res3 = ( intToInt32  (gc_res4)) in
  (return (((gc_res1,gc_res3,gc_res5,gc_res6),res2)))}
prim_clipBox :: ForeignObj -> IO (Ptr r1)
prim_clipBox a = withForeignPtr a prim_clipBox'
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_clipBox" prim_clipBox' :: ForeignObj' -> IO (Ptr r1)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_clipBox_gc_res2 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_clipBox_gc_res4 :: Ptr a1 -> IO (Int)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_clipBox_gc_res5 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_clipBox_gc_res6 :: Ptr a1 -> IO (Word32)
foreign import ccall unsafe "Xlib_stub_ffi.h" access_prim_clipBox_res2 :: Ptr a1 -> IO (Int)

----------------------------------------------------------------
-- Modifying regions
-- (If you use any of these, you can't make regions look like
--  first class data structures.)
----------------------------------------------------------------

-- translate region
offsetRegion :: Region -> Point -> IO Int
offsetRegion gc_arg1 gc_arg2 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg5) ->
  case ( int32ToInt  gc_arg3) of { gc_arg4 ->
  case ( int32ToInt  gc_arg5) of { gc_arg6 ->
  prim_offsetRegion arg1 gc_arg4 gc_arg6
  >>= \  res1  ->
  (return (res1))}}}}
prim_offsetRegion :: ForeignObj -> Int -> Int -> IO (Int)
prim_offsetRegion a b c = withForeignPtr a (\a' -> prim_offsetRegion' a' b c)
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_offsetRegion" prim_offsetRegion' :: ForeignObj' -> Int -> Int -> IO (Int)

-- increase size of region by +ve or -ve number of pixels
-- while preserving the centre of the region (ie half the pixels
-- come off the left, and half off the right)
shrinkRegion :: Region -> Point -> IO Int
shrinkRegion gc_arg1 gc_arg2 =
  case gc_arg1 of { (Region arg1) ->
  case gc_arg2 of { (gc_arg3,gc_arg5) ->
  case ( int32ToInt  gc_arg3) of { gc_arg4 ->
  case ( int32ToInt  gc_arg5) of { gc_arg6 ->
  prim_shrinkRegion arg1 gc_arg4 gc_arg6
  >>= \  res1  ->
  (return (res1))}}}}
prim_shrinkRegion :: ForeignObj -> Int -> Int -> IO (Int)
prim_shrinkRegion a b c = withForeignPtr a (\a' -> prim_shrinkRegion' a' b c)
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_shrinkRegion" prim_shrinkRegion' :: ForeignObj' -> Int -> Int -> IO (Int)

----------------------------------------------------------------
-- Graphics Context
----------------------------------------------------------------

-- set clip mask of GC
setRegion :: Display -> GC -> Region -> IO Int
setRegion gc_arg1 arg2 gc_arg2 =
  case gc_arg1 of { (Display arg1) ->
  case gc_arg2 of { (Region arg3) ->
  prim_setRegion arg1 arg2 arg3
  >>= \  res1  ->
  (return (res1))}}
prim_setRegion :: Addr -> Addr -> ForeignObj -> IO (Int)
prim_setRegion a b c = withForeignPtr c (\c' -> prim_setRegion' a b c')
foreign import  ccall unsafe "Xlib_stub_ffi.h prim_setRegion" prim_setRegion' :: Addr -> Addr -> ForeignObj' -> IO (Int)

----------------------------------------------------------------
-- End
----------------------------------------------------------------
